import eventlet
from flask import Flask, request, jsonify, session
from flask_sqlalchemy import SQLAlchemy
from flask_socketio import SocketIO, emit
from flask_httpauth import HTTPTokenAuth
from flask_login import current_user, LoginManager
import threading
from functools import wraps
from lxml import etree
from FreeTAKServer.model.FTSModel.Event import Event
from FreeTAKServer.model.RawCoT import RawCoT
from FreeTAKServer.controllers.ApplyFullJsonController import ApplyFullJsonController
from FreeTAKServer.controllers.XMLCoTController import XMLCoTController
from FreeTAKServer.model.ServiceObjects.FTS import FTS
from FreeTAKServer.controllers.configuration.RestAPIVariables import RestAPIVariables as vars
from FreeTAKServer.model.SimpleClient import SimpleClient
from FreeTAKServer.controllers.DatabaseControllers.DatabaseController import DatabaseController
from FreeTAKServer.controllers.configuration.DatabaseConfiguration import DatabaseConfiguration
from FreeTAKServer.controllers.RestMessageControllers.SendChatController import SendChatController
import os
import shutil
import json
from flask_cors import CORS
from FreeTAKServer.controllers.RestMessageControllers.SendSimpleCoTController import SendSimpleCoTController
from FreeTAKServer.controllers.RestMessageControllers.SendPresenceController import SendPresenceController
from FreeTAKServer.controllers.RestMessageControllers.SendEmergencyController import SendEmergencyController
from FreeTAKServer.controllers.configuration.MainConfig import MainConfig
from FreeTAKServer.controllers.JsonController import JsonController

dbController = DatabaseController()

UpdateArray = []
StartTime = None

functionNames = vars()
functionNames.function_names()

jsonVars = vars()
jsonVars.json_vars()

restMethods = vars()
restMethods.rest_methods()

defaultValues = vars()
defaultValues.default_values()

app = Flask(__name__)
login_manager = LoginManager()
login_manager.init_app(app)
auth = HTTPTokenAuth(scheme='Bearer')
app.config['SQLALCHEMY_DATABASE_URI'] = DatabaseConfiguration().DataBaseConnectionString
app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
db = SQLAlchemy(app)
dbController.session = db.session
CORS(app)
socketio = SocketIO(app, async_handlers=True, async_mode="eventlet")
socketio.init_app(app, cors_allowed_origins="*")
APIPipe = None
CommandPipe = None
app.config["SECRET_KEY"] = 'vnkdjnfjknfl1232#'


@app.errorhandler(404)
def page_not_found(e):
    return 'this endpoint does not exist'


@auth.verify_token
def verify_token(token):
    if token:
        output = dbController.query_APIUser(query=f'token == "{token}"')
        if output:
            return output[0].Username
        else:
            output = dbController.query_systemUser(query=f'token == "{token}"')
            if output:
                return output[0].name


@login_manager.user_loader
def load_user(user_id):
    return User.get(user_id)

def socket_auth(session = None):
    def innerfunc(x):
        def wrapper(*args, **kwargs):
            if hasattr(session, 'authenticated') and session.authenticated:
                x(*args, **kwargs)
            else:
                pass

        return wrapper

    return innerfunc


@app.route('/Alive')
def sessions():
    return b'API is running', 200


@socketio.on('connect')
def connection():
    emit('connectUpdate', json.dumps({"starttime": str(StartTime), "version": str(MainConfig.version)}))


@socketio.on('authenticate')
def authenticate(token):
    if json.loads(token)["Authenticate"] == MainConfig.websocketkey:
        emit('authentication', json.dumps({'successful': 'True'}))
        session.authenticated = True
    else:
        emit('authentication', json.dumps({'successful': 'False'}))


@socketio.on('users')
@socket_auth(session = session)
def show_users(empty=None):
    output = dbController.query_user()
    for i in range(0, len(output)):
        try:
            original = output[i]
            output[i] = output[i].__dict__
            print(output[i])
            try:
                output[i]['callsign'] = original.CoT.detail.contact.callsign
                output[i]['team'] = original.CoT.detail._group.name
            except:
                output[i]['callsign'] = "undefined"
                output[i]['team'] = "undefined"
            del (output[i]['_sa_instance_state'])
            del (output[i]['CoT_id'])
            del (output[i]['CoT'])
        except Exception as e:
            print(e)
    socketio.emit('userUpdate', json.dumps({"Users": output}))

@socketio.on('logs')
@socket_auth(session = session)
def return_logs(time):
    from FreeTAKServer.controllers.configuration.LoggingConstants import LoggingConstants
    import datetime
    log_data = {'log_data': []}
    for line in reversed(open(LoggingConstants().ERRORLOG, "r").readlines()):
        try:
            timeoflog = line.split(" : ")[1]
            if datetime.datetime.strptime(timeoflog, '%Y-%m-%d %H:%M:%S,%f') > datetime.datetime.strptime(
                    json.loads(time)["time"], '%Y-%m-%d %H:%M:%S,%f'):
                outline = {"time": '', "type": '', 'file': '', 'message': ''}
                line_segments = line.split(" : ")
                outline["type"] = line_segments[0]
                outline["time"] = line_segments[1]
                outline["file"] = line_segments[2]
                outline["message"] = " : ".join(line_segments[3:-1])
                log_data['log_data'].append(outline)
            else:
                break
        except:
            pass
    for num in range(1, 6):
        try:
            for line in reversed(open(LoggingConstants().ERRORLOG+'.'+str(num), "r").readlines()):
                try:
                    timeoflog = line.split(" : ")[1]
                    if datetime.datetime.strptime(timeoflog, '%Y-%m-%d %H:%M:%S,%f') > datetime.datetime.strptime(
                            json.loads(time)["time"], '%Y-%m-%d %H:%M:%S,%f'):
                        outline = {"time": '', "type": '', 'file': '', 'message': ''}
                        line_segments = line.split(" : ")
                        outline["type"] = line_segments[0]
                        outline["time"] = line_segments[1]
                        outline["file"] = line_segments[2]
                        outline["message"] = line_segments[3]
                        log_data['log_data'].append(outline)
                    else:
                        break
                except:
                    pass
        except:
            pass
    emit("logUpdate", json.dumps(log_data))


@socketio.on('serviceInfo')
@socket_auth(session = session)
def show_service_info(empty=None):
    mapping = {"start": "on", "stop": "off", "":""}
    FTSServerStatusObject = getStatus()
    tcpcot = FTSServerStatusObject.CoTService
    sslcot = FTSServerStatusObject.SSLCoTService
    restapi = FTSServerStatusObject.RestAPIService
    tcpdp = FTSServerStatusObject.TCPDataPackageService
    ssldp = FTSServerStatusObject.SSLDataPackageService
    fedserver = FTSServerStatusObject.FederationServerService
    jsonObject = {"services":
                      {"TCP_CoT_service": {"status": mapping[tcpcot.CoTServiceStatus], "port": tcpcot.CoTServicePort},
                       "SSL_CoT_service": {"status": mapping[sslcot.SSLCoTServiceStatus],
                                           "port": sslcot.SSLCoTServicePort},
                       "TCP_DataPackage_service": {"status": mapping[tcpdp.TCPDataPackageServiceStatus],
                                                   "port": tcpdp.TCPDataPackageServicePort},
                       "SSL_DataPackage_service": {"status": mapping[ssldp.SSLDataPackageServiceStatus],
                                                   "port": ssldp.SSLDataPackageServicePort},
                       "Federation_server_service": {"status": mapping[fedserver.FederationServerServiceStatus],
                                                     "port": fedserver.FederationServerServicePort},
                       "Rest_API_service": {"status": mapping[restapi.RestAPIServiceStatus],
                                            "port": restapi.RestAPIServicePort}},
                  "ip": tcpdp.TCPDataPackageServiceIP
                }
    emit('serviceInfoUpdate', json.dumps(jsonObject))

def getStatus():
    CommandPipe.send([functionNames.checkStatus])
    out = CommandPipe.recv()
    while hasattr(out, "CoTService") == False:
        out = CommandPipe.recv()
    return out
@socketio.on("serverHealth")
@socket_auth(session = session)
def serverHealth(empty=None):
    import psutil
    import pathlib
    import os
    jsondata = {
                "CPU": int(psutil.cpu_percent(interval=0.1)),
                "memory": int(psutil.virtual_memory().percent),
                "disk": int(psutil.disk_usage(str(pathlib.Path(os.getcwd()).anchor)).percent)
                }
    emit('serverHealthUpdate', json.dumps(jsondata))

@socketio.on('systemStatus')
@socket_auth(session=session)
def systemStatus(update=None):
    print('system status running')
    from FreeTAKServer.controllers.ServerStatusController import ServerStatusController
    currentStatus = getStatus()
    statusObject = ServerStatusController(currentStatus)
    jsondata = ApplyFullJsonController().serialize_model_to_json(statusObject)
    emit('systemStatusUpdate', json.dumps(jsondata))

@socketio.on('changeServiceInfo')
# @socket_auth(session=session)
def updateSystemStatus(update):
    # TODO: add documentation
    changeStatus(json.loads(update))
    show_service_info()

@socketio.on('systemUsers')
@socket_auth(session=session)
def systemUsers(empty=None):
    systemUserArray = DatabaseController().query_systemUser()
    jsondata = {"SystemUsers": []}
    for user in systemUserArray:
        userjson = {}
        userjson['Name'] = user.name
        userjson["Group"] = user.group
        userjson["Token"] = user.token
        userjson["Password"] = user.password
        userjson["Certs"] = user.certificate_package_name
        userjson["Uid"] = user.uid
        jsondata["SystemUsers"].append(userjson)

    emit('systemUsersUpdate', json.dumps(jsondata))

@socketio.on('addSystemUser')
@socket_auth(session=session)
def addSystemUser(jsondata):
    from FreeTAKServer.controllers import certificate_generation
    import uuid
    for systemuser in json.loads(jsondata)['systemUsers']:
        if systemuser["Certs"] == "true":
            # create certs
            certificate_generation.AtakOfTheCerts().bake(cn=systemuser["Name"])
            certificate_generation.generate_zip(user_filename=systemuser["Name"] + '.p12')
            # add DP
            import string
            import random
            from pathlib import PurePath, Path
            import hashlib
            from lxml import etree
            import shutil
            import os
            dp_directory = str(PurePath(Path(MainConfig.DataPackageFilePath)))
            openfile = open(str(PurePath(Path(str(MainConfig.clientPackages), systemuser["Name"] + '.zip'))), mode='rb')
            file_hash = str(hashlib.sha256(openfile.read()).hexdigest())
            openfile.close()
            newDirectory = str(PurePath(Path(dp_directory), Path(file_hash)))
            os.mkdir(newDirectory)
            shutil.copy(str(PurePath(Path(str(MainConfig.clientPackages), systemuser["Name"] + '.zip'))),
                        str(PurePath(Path(newDirectory), Path(systemuser["Name"] + '.zip'))))
            fileSize = Path(str(newDirectory), systemuser["Name"] + '.zip').stat().st_size
            dbController.create_datapackage(uid=str(uuid.uuid4()), Name=systemuser["Name"] + '.zip', Hash=file_hash,
                                            SubmissionUser='server',
                                            CreatorUid='server-uid', Size=fileSize, Privacy=1)
            DatabaseController().create_systemUser(name=systemuser["Name"], group=systemuser["Group"],
                                                   token=systemuser["Token"], password=systemuser["Password"],
                                                   uid=str(uuid.uuid4()), certificate_package_name=systemuser["Name"]+'.zip')
            import datetime as dt
            DATETIME_FMT = "%Y-%m-%dT%H:%M:%S.%fZ"
            timer = dt.datetime
            now = timer.utcnow()
            zulu = now.strftime(DATETIME_FMT)
            add = dt.timedelta(seconds=600)
            stale_part = dt.datetime.strptime(zulu, DATETIME_FMT) + add
            stale = stale_part.strftime(DATETIME_FMT)
            timer = dt.datetime
            now = timer.utcnow()
            zulu = now.strftime(DATETIME_FMT)
            time = zulu
            from FreeTAKServer.controllers.SpecificCoTControllers.SendOtherController import SendOtherController
            from FreeTAKServer.model.RawCoT import RawCoT
            cot = RawCoT()
            DPIP = getStatus().TCPDataPackageService.TCPDataPackageServiceIP
            clientXML = f'<?xml version="1.0"?><event version="2.0" uid="{str(uuid.uuid4())}" type="b-f-t-r" time="{time}" start="{time}" stale="{stale}" how="h-e"><point lat="43.85570300" lon="-66.10801200" hae="19.55866360" ce="3.21600008" le="nan" /><detail><fileshare filename="{systemuser["Name"]}" senderUrl="{DPIP}:8080/Marti/api/sync/metadata/{str(file_hash)}/tool" sizeInBytes="{fileSize}" sha256="{str(file_hash)}" senderUid="{"server-uid"}" senderCallsign="{"server"}" name="{systemuser["Name"]+".zip"}" /><ackrequest uid="{uuid.uuid4()}" ackrequested="true" tag="{systemuser["Name"]+".zip"}" /><marti><dest callsign="{systemuser["Name"]}" /></marti></detail></event>'
            cot.xmlString = clientXML.encode()
            newCoT = SendOtherController(cot)
            APIPipe.send(newCoT.getObject())

        else:
            DatabaseController().create_systemUser(name=systemuser["Name"], group=systemuser["Group"],
                                                   token=systemuser["Token"], password=systemuser["Password"],
                                                   uid=str(uuid.uuid4()))


@socketio.on("removeSystemUser")
@socket_auth(session=session)
def removeSystemUser(jsondata):
    jsondata = json.loads(jsondata)
    for systemUser in jsondata["systemUsers"]:
        uid = systemUser["uid"]
        systemUser = dbController.query_systemUser(query=f'uid == "{uid}"')[0]
        certificate_package_name = systemUser.certificate_package_name
        dbController.remove_systemUser(f'uid == "{uid}"')
        obj = dbController.query_datapackage(f'Name == "{certificate_package_name}"')
        # TODO: make this coherent with constants
        currentPath = MainConfig.DataPackageFilePath
        shutil.rmtree(f'{str(currentPath)}/{obj[0].Hash}')
        dbController.remove_datapackage(f'Hash == "{obj[0].Hash}"')

@socketio.on("events")
@socket_auth(session=session)
def events(empty=None):
    return socketio.emit("eventsUpdate", {"events": ["system user ALPHA created", "SSL DataPackage Service turned off", "TCP CoT Service port changed to 8086", "Outgoing Federation created to 1.1.1.1", "Federate connected from 0.0.0.0"]})

@app.route("/SendGeoChat", methods=[restMethods.POST])
@auth.login_required()
def SendGeoChat():
    try:
        json = request.json
        modelObject = Event.GeoChat()
        out = ApplyFullJsonController().serializeJsonToModel(modelObject, json)
        xml = XMLCoTController().serialize_model_to_CoT(out, 'event')
        from FreeTAKServer.controllers.SpecificCoTControllers.SendGeoChatController import SendGeoChatController
        rawcot = RawCoT()
        rawcot.xmlString = xml
        rawcot.clientInformation = None
        object = SendGeoChatController(rawcot)
        APIPipe.send(object.getObject())
        return '200', 200
    except Exception as e:
        print(e)



@app.route("/ManagePresence")
@auth.login_required()
def ManagePresence():
    pass

@app.route("/ManagePresence/postPresence", methods=[restMethods.POST])
@auth.login_required
def postPresence():
    try:
        from json import dumps
        #jsondata = {'longitude': '12.345678', 'latitude': '34.5677889', 'how': 'nonCoT', 'name': 'testing123'}
        jsondata = request.get_json(force=True)
        jsonobj = JsonController().serialize_presence_post(jsondata)
        Presence = SendPresenceController(jsonobj).getCoTObject()
        APIPipe.send(Presence)
        return Presence.modelObject.getuid(), 200
    except Exception as e:
        return str(e), 500

@app.route("/ManageGeoObject")
@auth.login_required()
def ManageGeoObject():
    pass

@app.route("/ManageGeoObject/postGeoObject", methods=[restMethods.POST])
@auth.login_required
def postGeoObject():
    try:
        from json import dumps
        #jsondata = {'longitude': '12.345678', 'latitude': '34.5677889', 'attitude': 'friend', 'geoObject': 'Ground', 'how': 'nonCoT', 'name': 'testing123'}
        jsondata = request.get_json(force=True)
        jsonobj = JsonController().serialize_geoobject_post(jsondata)
        simpleCoTObject = SendSimpleCoTController(jsonobj).getCoTObject()
        APIPipe.send(simpleCoTObject)
        return simpleCoTObject.modelObject.getuid(), 200
    except Exception as e:
        return str(e), 500

@app.route("/ManageChat")
@auth.login_required()
def ManageChat():
    pass

@app.route("/ManageChat/postChatToAll", methods=[restMethods.POST])
@auth.login_required
def postChatToAll():
    try:
        from json import dumps
        #jsondata = {'message': 'test abc', 'sender': 'natha'}
        jsondata = request.get_json(force=True)
        jsonobj = JsonController().serialize_chat_post(jsondata)
        ChatObject = SendChatController(jsonobj).getCoTObject()
        APIPipe.send(ChatObject)
        return 'success', 200
    except Exception as e:
        return str(e), 500

@app.route("/ManageEmergency/getEmergency", methods=[restMethods.GET])
@auth.login_required
def getEmergency():
    try:
        from json import dumps
        output = dbController.query_ActiveEmergency()
        for i in range(0, len(output)):
            original = output[i]
            output[i] = output[i].__dict__
            output[i]["lat"] = original.event.point.lat
            output[i]["lon"] = original.event.point.lon
            output[i]["type"] = original.event.detail.emergency.type
            output[i]["name"] = original.event.detail.contact.callsign
            del (output[i]['_sa_instance_state'])
            del(output[i]['event'])
        return jsonify(json_list=output), 200
    except Exception as e:
        return str(e), 200

@app.route("/ManageEmergency/postEmergency", methods=[restMethods.POST])
@auth.login_required
def postEmergency():
    try:
        from json import dumps

        jsondata = request.get_json(force=True)
        jsonobj = JsonController().serialize_emergency_post(jsondata)
        EmergencyObject = SendEmergencyController(jsonobj).getCoTObject()
        APIPipe.send(EmergencyObject)
        return EmergencyObject.modelObject.getuid(), 200
    except Exception as e:
        return str(e), 200

@app.route("/ManageEmergency/deleteEmergency", methods=[restMethods.DELETE])
@auth.login_required
def deleteEmergency():
    try:
        from json import dumps
        jsondata = request.get_json(force=True)
        jsonobj = JsonController().serialize_emergency_delete(jsondata)
        EmergencyObject = SendEmergencyController(jsonobj).getCoTObject()
        APIPipe.send(EmergencyObject)
        return 'success', 200
    except Exception as e:
        return str(e), 500

@app.route("/ManageEmergency")
@auth.login_required
def Emergency():
    pass

#@app.route("/ConnectionMessage", methods=[restMethods.POST])
def ConnectionMessage():

    try:
        json = request.json
        modelObject = Event.GeoChat()
        out = ApplyFullJsonController().serializeJsonToModel(modelObject, json)
        xml = XMLCoTController().serialize_model_to_CoT(out, 'event')
        from FreeTAKServer.controllers import SendGeoChatController
        rawcot = RawCoT()
        rawcot.xmlString = xml
        rawcot.clientInformation = None
        object = SendGeoChatController(rawcot).getObject()
        object.type = "connmessage"
        APIPipe.send(object.SendGeoChat)
        return '200', 200
    except Exception as e:
        print(e)

@app.route("/APIUser", methods=[restMethods.GET, restMethods.POST, restMethods.DELETE])
def APIUser():
    if request.remote_addr in MainConfig.AllowedCLIIPs:
        try:
            if request.method == restMethods.POST:
                json = request.get_json()
                dbController.create_APIUser(Username=json['username'], Token=json['token'])
                return 'success', 200

            elif request.method == restMethods.DELETE:
                json = request.get_json()
                username = json['username']
                dbController.remove_APIUser(query=f'Username == "{username}"')
                return 'success', 200

            elif request.method == restMethods.GET:
                output = dbController.query_APIUser()
                for i in range(0, len(output)):
                    output[i] = output[i].__dict__
                    del (output[i]['_sa_instance_state'])
                    del (output[i]['PrimaryKey'])
                    del (output[i]['uid'])
                return jsonify(json_list=output), 200

        except Exception as e:
            return str(e), 500
    else:
        return 'endpoint can only be accessed by approved IPs', 401
@app.route("/RecentCoT", methods=[restMethods.GET])
def RecentCoT():
    import time
    time.sleep(10)
    return b'1234'

@app.route("/URL", methods=[restMethods.GET])
def URLGET():
    data = request.args
    print(data)
    return 'completed', 200

@app.route("/Clients", methods=[restMethods.GET])
def Clients():
    try:
        if request.remote_addr in MainConfig.AllowedCLIIPs:
            CommandPipe.send([functionNames.Clients])
            out = CommandPipe.recv()
            returnValue = []
            for client in out:
                returnValue.append(ApplyFullJsonController().serialize_model_to_json(client))
            dumps = json.dumps(returnValue)
            return dumps
        else:
            return 'endpoint can only be accessed by approved IPs', 401
    except Exception as e:
        return str(e), 500

@app.route('/FederationTable', methods=[restMethods.GET, restMethods.POST, "PUT", restMethods.DELETE])
@auth.login_required()
def FederationTable():
    try:
        import random
        if request.method == restMethods.GET:
            output = dbController.query_ActiveFederation()
            for i in range(0, len(output)):
                output[i] = output[i].__dict__
                del (output[i]['_sa_instance_state'])
            output2 = dbController.query_Federation()
            for i in range(0, len(output2)):
                output2[i] = output2[i].__dict__
                del (output2[i]['_sa_instance_state'])

            return jsonify(activeFederations=output, federations=output2), 200

        elif request.method == restMethods.POST:
            import uuid
            jsondata = json.loads(request.data)
            new_outgoing_federations = jsondata["outgoingFederations"]
            for new_fed in new_outgoing_federations:
                id = str(uuid.uuid4())
                dbController.create_Federation(**new_fed, id = id)
                if new_fed["status"] == "Enabled":
                    CommandPipe.send((id, "CREATE"))
                else:
                    pass
            return '', 200

        elif request.method == "PUT":
            jsondata = json.loads(request.data)
            federations = jsondata["federations"]
            for fed in federations:
                old_fed = dbController.query_Federation(f'id == "{fed["id"]}"')[0]
                old_status = old_fed.status
                old_id = old_fed.id
                dbController.update_Federation(column_value=fed, query=f'id == "{fed["id"]}"')
                if "status" in fed:
                    if fed["status"] == "Enabled":

                        if old_status != "Enabled":
                            CommandPipe.send((old_id, "CREATE"))
                        else:
                            pass
                    elif fed["status"] == "Disabled":
                        if old_status != "Disabled":
                            CommandPipe.send((old_id, "DELETE"))
                        else:
                            pass
                    else:
                        pass
            return '', 200

        elif request.method == "DELETE":
            jsondata = json.loads(request.data)
            federations = jsondata["federations"]
            for fed in federations:
                fedInstance = dbController.query_Federation(f'id == "{fed["id"]}"')[0]
                if fedInstance.status == "Enabled":
                    CommandPipe.send((fedInstance.id, "DELETE"))
                else:
                    pass
                dbController.remove_Federation(f'id == "{fed["id"]}"')
            return '', 200

    except Exception as e:
        return str(e), 500

@app.route('/DataPackageTable', methods=[restMethods.GET, restMethods.POST, restMethods.DELETE, "PUT"])
@auth.login_required()
def DataPackageTable():
    from pathlib import Path
    if request.method == "GET":
        output = dbController.query_datapackage()
        for i in range(0, len(output)):
            output[i] = output[i].__dict__
            del (output[i]['_sa_instance_state'])
            del (output[i]['CreatorUid'])
            del (output[i]['MIMEType'])
            del (output[i]['uid'])
        return jsonify(json_list=output), 200

    elif request.method == "DELETE":
        jsondata = json.loads(request.data)
        Hashes = jsondata['DataPackages']
        for hash in Hashes:
            Hash = hash['hash']
            print(Hash)
            obj = dbController.query_datapackage(f'Hash == "{Hash}"')
            print(obj)
            # TODO: make this coherent with constants
            currentPath = MainConfig.DataPackageFilePath
            shutil.rmtree(f'{str(currentPath)}/{obj[0].Hash}')
            dbController.remove_datapackage(f'Hash == "{Hash}"')
        return '200', 200

    elif request.method == "POST":
        import string
        import random
        from pathlib import PurePath, Path
        import hashlib
        from zipfile import ZipFile, ZIP_DEFLATED
        import tempfile
        from lxml import etree
        import uuid
        from FreeTAKServer.controllers.configuration.DataPackageServerConstants import DataPackageServerConstants
        dp_directory = str(PurePath(Path(MainConfig.DataPackageFilePath)))
        letters = string.ascii_letters
        #uid = ''.join(random.choice(letters) for i in range(4))
        #uid = 'uid-' + str(uid)
        uid = str(uuid.uuid4())
        filename = request.args.get('filename')
        creatorUid = request.args.get('creatorUid')
        file = request.files.getlist('assetfile')[0]
        with ZipFile(file, mode='a') as zip:
            if "MANIFEST/manifest.xml" in [member.filename for member in zip.infolist()]:
                manifestXML = etree.Element("MissionPackageManifest", version="2")
                config = etree.SubElement(manifestXML, "Configuration")
                etree.SubElement(config, "Parameter", name="uid", value=uid)
                etree.SubElement(config, "Parameter", name="name", value=filename)

                contents = etree.SubElement(manifestXML, "Contents")
                for fileName in zip.namelist():
                    etree.SubElement(contents, "Content", ignore="false", zipEntry=str(fileName))
                # manifest = zip.open('MANIFEST\\manifest.xml', mode="w")
                zip.writestr('MANIFEST\\manifest.xml', etree.tostring(manifestXML))
                print(zip.namelist())
                file.seek(0)
            else:
                pass

        tempuid = str(uuid.uuid4())
        app.logger.info(f"Data Package hash = {str(tempuid)}")
        directory = Path(dp_directory, tempuid)
        if not Path.exists(directory):
            os.mkdir(str(directory))
        file.seek(0)
        filepath = str(PurePath(Path(directory), Path(filename)))
        file.save(filepath)
        openfile = open(str(PurePath(Path(str(directory), filename))), mode='rb')
        file_hash = str(hashlib.sha256(openfile.read()).hexdigest())
        openfile.close()
        newDirectory = str(PurePath(Path(dp_directory), Path(file_hash)))
        os.rename(str(PurePath(Path(directory))), newDirectory)
        fileSize = Path(str(newDirectory), filename).stat().st_size
        if creatorUid == None:
            callsign = str(dbController.query_user(query=f'uid == "server-uid"', column=[
                'callsign']))  # fetchone() gives a tuple, so only grab the first element
            dbController.create_datapackage(uid=uid, Name=filename, Hash=file_hash, SubmissionUser='server',
                                            CreatorUid='server-uid', Size=fileSize)
        else:
            callsign = str(dbController.query_user(query=f'uid == f"{creatorUid}"', column=[
                'callsign']))  # fetchone() gives a tuple, so only grab the first element
            dbController.create_datapackage(uid=uid, Name=filename, Hash=file_hash, SubmissionUser=callsign,
                                            CreatorUid=creatorUid, Size=fileSize)
        return 'successful', 200

    elif request.method == "PUT":
        updatedata = json.loads(request.data)
        DataPackages = updatedata['DataPackages']
        for dp in DataPackages:
            updateDict = {}
            if 'Privacy' in dp:
                updateDict["Privacy"] = int(dp["Privacy"])
            if "Keywords" in dp:
                updateDict["Keywords"] = dp["Keywords"]
            if "Name" in dp:
                updateDict["Name"] = dp["Name"]
            dbController.update_datapackage(query=f'PrimaryKey == {dp["PrimaryKey"]}', column_value=updateDict)
        return "success", 200
def getMission():
    import uuid
    import random
    creator_uid = str(uuid.uuid4())
    maincontent = {
        "name": "some name",
        "description": "some description",
        "chatRoom": "",
        "tool": "public",
        "keywords": ["optional", "set", "of", "keywords"],
        "creatorUid": creator_uid,
        "createTime": "2020-12-09T15:53:42.873Z",
        "groups": ["Allowed", "groups"],
        "externalData": [],
        "uids": [{
            "data": str(uuid.uuid4()),
            "timestamp": "2020-12-09T15:58:10.635Z",
            "creatorUid": creator_uid,
            "details": {
                "type": "a-h-G",
                "callsign": "R.9.155734",
                "iconsetPath": "COT_MAPPING_2525B/a-h/a-h-G"
            }
        }
        ],
        "contents": [{
            "data": {
                "filename": "name of file",
                "keywords": [],
                "mimeType": "application/octet-stream",
                "name": "name of mission",
                "submissionTime": "2020-12-09T15:55:21.468Z",
                "submitter": "submitter name",
                "uid": str(uuid.uuid4()),
                "hash": str(random.getrandbits(128)),
                "size": random.randint(1000, 100000)
            },
            "timestamp": "2020-12-09T15:55:21.559Z",
            "creatorUid": creator_uid
        }
        ],
        "passwordProtected": random.choice(['true', 'false'])
    }
    return maincontent


@app.route("/MissionTable", methods=['GET', 'POST', 'DELETE'])
@auth.login_required()
def mission_table():
    try:
        if request.method == "GET":
            import random

            jsondata = {
                "version": "3",
                "type": "Mission",
                "data": [],
                "nodeId": "6ff99444fa124679a3943ee90308a44c9d794c02-e5a5-42b5-b4c8-625203ea1287"
            }
            return json.dumps(jsondata)
        elif request.method == "POST":
            return b'', 200
        elif request.method == "DELETE":
            return b'', 200
    except Exception as e:
        return e, 500

@app.route("/ExCheckTable", methods=["GET", "POST", "DELETE"])
@auth.login_required()
def excheck_table():
    try:
        from os import listdir
        from pathlib import PurePath, Path
        from datetime import datetime
        from flask import request
        if request.method == "GET":
            jsondata = {"ExCheck":{'Templates': [], 'Checklists': []}}
            from FreeTAKServer.controllers.ExCheckControllers.templateToJsonSerializer import templateSerializer
            excheckTemplates = DatabaseController().query_ExCheck()
            for template in excheckTemplates:
                templateData = template.data
                templatejson = 	{
                                    "filename": templateData.filename,
                                    "name": templateData.keywords.name,
                                    "submissionTime": templateData.submissionTime,
                                    "submitter": str(dbController.query_user(query=f'uid == "{template.creatorUid}"', column=['callsign'])),
                                    "uid": templateData.uid,
                                    "hash": templateData.hash,
                                    "size": templateData.size,
                                    "description": templateData.keywords.description
                                }
                jsondata["ExCheck"]['Templates'].append(templatejson)
            excheckChecklists = DatabaseController().query_ExCheckChecklist()
            for checklist in excheckChecklists:
                try:
                    templatename = checklist.template.data.name
                except AttributeError:
                    templatename = "template removed"
                checklistjson = {
                                    "filename": checklist.filename,
                                    "name": checklist.name,
                                    "startTime": datetime.strftime(checklist.startTime, "%Y-%m-%dT%H:%M:%S.%fZ"),
                                    "submitter": checklist.callsign,
                                    "uid": checklist.uid,
                                    "description": checklist.description,
                                    "template": templatename
                                }
                jsondata["ExCheck"]['Checklists'].append(checklistjson)
            return json.dumps(jsondata), 200

        elif request.method == "DELETE":
            jsondata = request.data
            ExCheckArray = json.loads(jsondata)["ExCheck"]
            for item in ExCheckArray["Templates"]:
                templateitem = DatabaseController().query_ExCheck(f'ExCheckData.uid == "{item["uid"]}"', verbose=True)[0]
                os.remove(str(PurePath(Path(MainConfig.ExCheckFilePath), Path(templateitem.data.filename))))
                DatabaseController().remove_ExCheck(f'PrimaryKey == "{templateitem.PrimaryKey}"')
            for item in ExCheckArray["Checklists"]:
                checklistitem = DatabaseController().query_ExCheckChecklist(f'uid == "{item["uid"]}"')[0]
                os.remove(str(PurePath(Path(MainConfig.ExCheckChecklistFilePath), Path(checklistitem.filename))))
                DatabaseController().remove_ExCheckChecklist(f'uid == "{item["uid"]}"')
            return 'success', 200
        elif request.method == "POST":
            try:
                import uuid
                from FreeTAKServer.controllers.ExCheckControllers.templateToJsonSerializer import templateSerializer
                xmlstring = f'<?xml version="1.0"?><event version="2.0" uid="{uuid.uuid4()}" type="t-x-m-c" time="2020-11-28T17:45:51.000Z" start="2020-11-28T17:45:51.000Z" stale="2020-11-28T17:46:11.000Z" how="h-g-i-g-o"><point lat="0.00000000" lon="0.00000000" hae="0.00000000" ce="9999999" le="9999999" /><detail><mission type="CHANGE" tool="ExCheck" name="exchecktemplates" authorUid="S-1-5-21-2720623347-3037847324-4167270909-1002"><MissionChanges><MissionChange><contentResource><filename>61b01475-ad44-4300-addc-a9474ebf67b0.xml</filename><hash>018cd5786bd6c2e603beef30d6a59987b72944a60de9e11562297c35ebdb7fd6</hash><keywords>test init</keywords><keywords>dessc init</keywords><keywords>FEATHER</keywords><mimeType>application/xml</mimeType><name>61b01475-ad44-4300-addc-a9474ebf67b0</name><size>1522</size><submissionTime>2020-11-28T17:45:47.980Z</submissionTime><submitter>wintak</submitter><tool>ExCheck</tool><uid>61b01475-ad44-4300-addc-a9474ebf67b0</uid></contentResource><creatorUid>S-1-5-21-2720623347-3037847324-4167270909-1002</creatorUid><missionName>exchecktemplates</missionName><timestamp>2020-11-28T17:45:47.983Z</timestamp><type>ADD_CONTENT</type></MissionChange></MissionChanges></mission></detail></event>'
                # this is where the client will post the xmi of a template
                from datetime import datetime
                from lxml import etree
                import hashlib
                # possibly the uid of the client submitting the template
                authoruid = request.args.get('clientUid')
                if not authoruid:
                    authoruid = 'server-uid'
                XMI = request.data.decode()
                serializer = templateSerializer(XMI)
                object = serializer.convert_template_to_object()
                object.timestamp = datetime.strptime(object.timestamp, "%Y-%m-%dT%H:%M:%S.%fZ")
                serializer.create_DB_object(object)
                xml = etree.fromstring(XMI)
                path = str(PurePath(Path(MainConfig.ExCheckFilePath), Path(f'{object.data.uid}.xml')))
                with open(path, 'w+') as file:
                    file.write(XMI)
                    file.close()

                uid = object.data.uid
                temp = etree.fromstring(XMI)
                cot = etree.fromstring(xmlstring)
                cot.find('detail').find('mission').set("authorUid", authoruid)
                resources = cot.find('detail').find('mission').find('MissionChanges').find('MissionChange').find(
                    'contentResource')
                resources.find('filename').text = temp.find('checklistDetails').find('uid').text + '.xml'
                resources.findall('keywords')[0].text = temp.find('checklistDetails').find('name').text
                resources.findall('keywords')[1].text = temp.find('checklistDetails').find('description').text
                resources.findall('keywords')[2].text = temp.find('checklistDetails').find('creatorCallsign').text
                resources.find('uid').text = temp.find('checklistDetails').find('uid').text
                resources.find('name').text = temp.find('checklistDetails').find('uid').text
                resources.find('size').text = str(len(XMI))
                resources.find('hash').text = str(hashlib.sha256(str(XMI).encode()).hexdigest())
                z = etree.tostring(cot)
                from FreeTAKServer.model.testobj import testobj
                object = testobj()
                object.xmlString = z
                APIPipe.send(object)
                return str(uid), 200
            except Exception as e:
                print(str(e))
    except Exception as e:
        return str(e), 500
@app.route('/checkStatus', methods=[restMethods.GET])
@auth.login_required()
def check_status():
    try:
        if request.remote_addr in MainConfig.AllowedCLIIPs:
            CommandPipe.send([functionNames.checkStatus])
            FTSServerStatusObject = CommandPipe.recv()
            out = ApplyFullJsonController().serialize_model_to_json(FTSServerStatusObject)
            return json.dumps(out), 200
        else:
            return 'endpoint can only be accessed by approved IPs', 401
    except Exception as e:
        return str(e), 500

#@app.route('/changeStatus', methods=[restMethods.POST])
#@auth.login_required()
def changeStatus(jsonmessage):
    # TODO: modify to better support format
    mappings = {"on": "start", "off": "stop", "": ""}
    try:
        import json
        if not jsonmessage:
            jsonmessage = json.loads(request.data)
        FTSObject = FTS()
        json = jsonmessage["services"]
        ip = jsonmessage.get("ip")
        if jsonVars.COTSERVICE in json:
            CoTService = json[jsonVars.COTSERVICE]
            try:
                FTSObject.CoTService.CoTServicePort = int(CoTService.get(jsonVars.PORT))
            except:
                FTSObject.CoTService.CoTServicePort = ''
            FTSObject.CoTService.CoTServiceStatus = mappings[CoTService.get("status")]
        else:
            pass

        if jsonVars.DATAPACKAGESERVICE in json:

            DPService = json.get(jsonVars.DATAPACKAGESERVICE)
            try:
                FTSObject.TCPDataPackageService.TCPDataPackageServicePort = int(DPService.get(jsonVars.PORT))
            except:
                FTSObject.TCPDataPackageService.TCPDataPackageServicePort = ''
            FTSObject.TCPDataPackageService.TCPDataPackageServiceStatus = mappings[DPService.get("status")]

        else:
            pass

        if jsonVars.SSLDATAPACKAGESERVICE in json:

            DPService = json.get(jsonVars.SSLDATAPACKAGESERVICE)
            try:
                FTSObject.SSLDataPackageService.SSLDataPackageServicePort = int(DPService.get(jsonVars.PORT))
            except:
                FTSObject.SSLDataPackageService.SSLDataPackageServicePort = ''
            FTSObject.SSLDataPackageService.SSLDataPackageServiceStatus = mappings[DPService.get("status")]

        else:
            pass

        if jsonVars.SSLCOTSERVICE in json:

            SSLCoTservice = json[jsonVars.SSLCOTSERVICE]
            try:
                FTSObject.SSLCoTService.SSLCoTServicePort = int(SSLCoTservice.get(jsonVars.PORT))
            except:
                FTSObject.SSLCoTService.SSLCoTServicePort = ''
            FTSObject.SSLCoTService.SSLCoTServiceStatus = mappings[SSLCoTservice.get("status")]

        else:
            pass

        if jsonVars.FEDERATIONSERVERSERVICE in json:

            FederationServerService = json.get(jsonVars.FEDERATIONSERVERSERVICE)
            try:
                FTSObject.FederationServerService.FederationServerServicePort = int(FederationServerService.get(jsonVars.PORT))
            except:
                FTSObject.FederationServerService.FederationServerServicePort = ''
            FTSObject.FederationServerService.FederationServerServiceStatus = mappings[FederationServerService.get('status')]

        else:
            pass

        if jsonVars.RESTAPISERVICE in json:

            RESTAPISERVICE = json.get(jsonVars.RESTAPISERVICE)
            try:
                FTSObject.RestAPIService.RestAPIServicePort = int(RESTAPISERVICE.get(jsonVars.PORT))
            except:
                FTSObject.RestAPIService.RestAPIServicePort = ''
            FTSObject.RestAPIService.RestAPIServiceStatus = mappings[RESTAPISERVICE.get("status")]

        else:
            pass
        FTSObject.SSLDataPackageService.SSLDataPackageServiceIP = ip
        FTSObject.TCPDataPackageService.TCPDataPackageServiceIP = ip
        CommandPipe.send([functionNames.Status, FTSObject])
        out = CommandPipe.recv()
        return '200', 200

    except Exception as e:
        return '500', 500

def receiveUpdates():
    while True:
        try:
            update = APIPipe.recv()
            global UpdateArray
            UpdateArray.append(update)
        except Exception as e:
            print(e)

def submitData(dataRaw):
    global APIPipe
    print(APIPipe)
    data = RawCoT()
    data.clientInformation = "SERVER"
    data.xmlString = dataRaw.encode()
    APIPipe.send([data])

def emitUpdates(Updates):
    data = [SimpleClient()]
    data[0].callsign = ''
    data[0].team = ''
    data[0].ip = ''
    returnValue = []
    for client in data:
        returnValue.append(ApplyFullJsonController().serialize_model_to_json(client))
    socketio.emit('up', json.dumps(returnValue), broadcast = True)
    data = Updates
    for client in data:
        returnValue.append(ApplyFullJsonController().serialize_model_to_json(client))
    socketio.emit('up', json.dumps(returnValue), broadcast = True)
    return 1

def test(json):
    modelObject = Event.dropPoint()
    out = XMLCoTController().serialize_CoT_to_model(modelObject, etree.fromstring(json))
    xml = XMLCoTController().serialize_model_to_CoT(out, 'event')
    from FreeTAKServer.controllers.SpecificCoTControllers.SendDropPointController import SendDropPointController
    rawcot = RawCoT()
    rawcot.xmlString = xml
    rawcot.clientInformation = None
    object = SendDropPointController(rawcot)
    print(etree.tostring(object.sendDropPoint.xmlString,pretty_print=True).decode())
    '''EventObject = json
    modelObject = ApplyFullJsonController(json, 'Point', modelObject).determine_function()
    out = XMLCoTController().serialize_model_to_CoT(modelObject, 'event')
    print(etree.tostring(out))
    print(RestAPI().serializeJsonToModel(modelObject, EventObject))'''



class RestAPI:
    def __init__(self):
        pass

    def startup(self, APIPipea, CommandPipea, IP, Port, starttime):
        print('running api')
        global APIPipe, CommandPipe, StartTime
        StartTime = starttime
        APIPipe = APIPipea
        CommandPipe = CommandPipea
        threading.Thread(target=receiveUpdates, daemon=True, args=()).start()
        socketio.run(app, host=IP, port=Port)
        # try below if something breaks
        # socketio.run(app, host='0.0.0.0', port=10984, debug=True, use_reloader=False)


    def serializeJsonToModel(self, model, Json):
        for key, value in Json.items():
            if isinstance(value, dict):
                submodel = getattr(model, key)
                out = self.serializeJsonToModel(submodel, value)
                setattr(model, key, out)
            else:
                setattr(model, key, value)
        return model

if __name__ == '__main__':
    excheck_table()
    #    app.run(host="127.0.0.1", port=80)



#######################################################
# 
# ActiveThreads.py
# Python implementation of the Class ActiveThreads
# Generated by Enterprise Architect
# Created on:      21-May-2020 10:18:14 AM
# Original author: Natha Paquette
# 
#######################################################


class ActiveThreads:
# default constructor  def __init__(self):  

    ThreadArray = []



#######################################################
# 
# ClientInformationQueue.py
# Python implementation of the Class ClientInformationQueue
# Generated by Enterprise Architect
# Created on:      21-May-2020 9:47:34 AM
# Original author: Natha Paquette
# 
#######################################################
from multiprocessing import Queue

class ClientInformationQueue:
    def __init__(self):  

        self.clientQueue = Queue()


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
#######################################################
# 
# Precisionlocation.py
# Python implementation of the Class Precisionlocation
# Generated by Enterprise Architect
# Created on(FTSProtocolObject):      11-Feb-2020 11(FTSProtocolObject):08(FTSProtocolObject):10 AM
# Original author: Corvo
# 
#######################################################
from FreeTAKServer.model.FTSModelVariables.PrecisionlocationVariables import PrecisionlocationVariables as vars

class Precisionlocation(FTSProtocolObject):
    """some type of location?
    """
    
# TDB can be DTED0 or ???
 

    # default constructor       
    def __init__(self):
        self.altsrc = None
        self.geopointsrc = None

    @staticmethod
    def drop_point(ALTSRC = vars.drop_point().ALTSRC):
        precisionlocation = Precisionlocation()
        precisionlocation.setaltsrc(altsrc=ALTSRC)
        return precisionlocation

    @staticmethod
    def connection(ALTSRC = vars.connection().ALTSRC, GEOPOINTSRC = vars.connection().GEOPOINTSRC):
        precisionlocation = Precisionlocation()
        precisionlocation.setaltsrc(altsrc=ALTSRC)
        precisionlocation.setgeopointsrc(geopointsrc=GEOPOINTSRC)
        return precisionlocation

    @staticmethod
    def UserUpdate(ALTSRC=vars.UserUpdate().altsrc, GEOPOINTSRC=vars.UserUpdate().geopointsrc):
        precisionlocation = Precisionlocation()
        precisionlocation.setaltsrc(ALTSRC)
        precisionlocation.setgeopointsrc(GEOPOINTSRC)
        return precisionlocation

    # geopointsrc getter
    def getgeopointsrc(self): 
      return self.geopointsrc 
 
     # geopointsrc setter 
    def setgeopointsrc(self, geopointsrc=0):  
        self.geopointsrc=geopointsrc 
    
          # altsrc getter 
    def getaltsrc(self): 
        return self.altsrc 
 
     # altsrc setter 
    def setaltsrc(self, altsrc=0):  
        self.altsrc=altsrc 
 


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
from FreeTAKServer.model.FTSModel.ChecklistColumns import ChecklistColumns
from FreeTAKServer.model.FTSModel.ChecklistDetails import ChecklistDetails
from FreeTAKServer.model.FTSModel.ChecklistTasks import ChecklistTasks

class Checklist(FTSProtocolObject):
    @staticmethod
    def Checklist():
        checklist = Checklist()
        checklist.checklistDetails = ChecklistDetails.Checklist()
        checklist.checklistColumns = ChecklistColumns.Checklist()
        checklist.checklistTasks = ChecklistTasks.Checklist()
        return checklist


#######################################################
# 
# Point.py
# Python implementation of the Class Point
# Generated by Enterprise Architect
# Created on:      26-Sep-2020 9:41:22 PM
# Original author: natha
# 
#######################################################
from sqlalchemy import Column
from FreeTAKServer.model.SQLAlchemy.Root import Base
from sqlalchemy import String
from sqlalchemy import ForeignKey


class Point(Base):
# default constructor  def __init__(self):  
    __tablename__ = "Point"
    PrimaryKey = Column(ForeignKey("Event.uid"), primary_key=True)
    ce = Column(String)
    hae = Column(String)
    lat = Column(String)
    le = Column(String)
    lon = Column(String)


from FreeTAKServer.controllers.DatabaseControllers.table_controllers import TableController
from FreeTAKServer.model.SQLAlchemy.system_user import SystemUser

class SystemUserTableController(TableController):
    def __init__(self):
        self.table = SystemUser


class ClientInterfaceController:
    def __init__(self):
        self.commandDict = {
            'help': self.helpCommand
        }

    def commandInput(self):
        command = str(input('FTS $ '))
        output = self.commandDict[command]()
        print(output)

    def helpCommand(self):
        output = "this is currently just astetic"
        return output
ClientInterfaceController().commandInput()


#######################################################
# 
# ReceiveConnectionsProcess.py
# Python implementation of the Class ReceiveConnectionsProcess
# Generated by Enterprise Architect
# Created on:      21-May-2020 9:47:41 AM
# Original author: Natha Paquette
# 
#######################################################

class ReceiveConnectionsProcess:
    def __init__(self):  
        self.receiveConnections = ""



#######################################################
# 
# StopHandler.py
# Python implementation of the Class StopHandler
# Generated by Enterprise Architect
# Created on:      29-Dec-2020 8:10:47 AM
# Original author: natha
# 
#######################################################
from Catalog.Data.Federation.Handler import Handler

class StopHandler(Handler):
# default constructor  def __init__(self):  

    def Handle(object, command):
        pass

    def setNextHandler(handler):
        pass


from FreeTAKServer.model.RestMessages.GeoObject import GeoObject, RestEnumerations

class GeoObjectPost(GeoObject):
    def __init__(self):
        pass
    
    def setattitude(self, attitude):
        self.attitude = RestEnumerations.attitude[attitude]

    def getattitude(self):
        return self.attitude

    def setgeoObject(self, geoObject):
        self.geoObject = RestEnumerations.supportedTypeEnumerations[geoObject]

    def getgeoObject(self):
        return self.geoObject

    def sethow(self, how):
        self.how = RestEnumerations.how[how]

    def gethow(self):
        return self.how

    def setlatitude(self, latitude):
        self.latitude = str(latitude)

    def getlatitude(self):
        return self.latitude

    def setlongitude(self, longitude):
        self.longitude = str(longitude)

    def getlongitude(self):
        return self.longitude

    def setname(self, name):
        self.name = str(name)

    def getname(self):
        return self.name

    def settimeout(self, timeout):
        self.timeout = int(timeout)

    def gettimeout(self):
        return self.timeout




from abc import ABC, abstractmethod
from typing import NewType, List
from FreeTAKServer.model.FTSModel.Event import Event
from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
class SerializerAbstract(ABC):
    """ logic for interacting with FTSObjects and methods which should be available in all serializers"""

    __ftsObjectType = NewType('ftsObject', FTSProtocolObject)

    @abstractmethod
    def from_format_to_fts_object(self, object: type, FTSObject: Event) -> Event:
        raise NotImplementedError

    @abstractmethod
    def from_fts_object_to_format(self, FTSObject: Event) -> type:
        raise NotImplementedError

    @abstractmethod
    def _get_method_in_method_list(self, method_list: List[callable], expected_class_name: str) -> callable:
        raise NotImplementedError

    def _get_fts_object_var_setter(self, FTSObject: __ftsObjectType, variable_name: str, setterlist=None) -> list:
        """ return variable setter from object

            recursively search the provided FTSObject instance for
            the setter associated with the specified variable which will
            then be returned. if this is not found an attribute error will
            be raised
        """

        if setterlist is None:
            setterlist = []
        variables = vars(FTSObject)
        for key, value in variables.items():
            if '_'+FTSObject.__class__.__name__.lower()+'__' in key.lower():
                continue
            elif issubclass(type(value), FTSProtocolObject):
                getter = getattr(FTSObject, "get"+key)
                setterlist.extend(self._get_fts_object_var_setter(getter(), variable_name))
            elif isinstance(value, list):
                getter = getattr(FTSObject, "get" + key)
                setterlist.extend(self._get_fts_object_var_setter(getter(), variable_name))
            elif variable_name == key:
                setterlist.append(getattr(FTSObject, 'set' + key))
            else:
                pass

        return setterlist

    def _get_fts_object_var_getter(self, FTSObject: __ftsObjectType, variable_name: str, getterlist = None) -> list:
        """ return variable getter from object

            recursively search the provided FTSObject instance for
            the getter associated with the specified variable which will
            then be returned. if this is not available an attribute error will
            be raised
        """
        if getterlist is None:
            getterlist = []

        variables = vars(FTSObject)
        for key, value in variables.items():
            if '_'+FTSObject.__class__.__name__.lower()+'__' in key.lower():
                continue
            elif issubclass(type(value), FTSProtocolObject):
                getterlist.extend(self._get_fts_object_var_getter(value, variable_name))
            elif isinstance(value, list):
                getter = getattr(FTSObject, 'get'+key)
                getterlist.extend(self._get_fts_object_var_getter(getter(), variable_name))
            elif variable_name == key:
                getterlist.append(getattr(FTSObject, 'get' + variable_name))
            else:
                pass

        return getterlist


from FreeTAKServer.model.RestMessages.Emergency import Emergency, RestEnumerations

class EmergencyDelete(Emergency):
    def __init__(self):
        pass

    def setuid(self, uid):
        self.uid = uid

    def getuid(self):
        return self.uid


from FreeTAKServer.model.RestMessages.Emergency import Emergency, RestEnumerations

class EmergencyPost(Emergency):
    def __init__(self):
        pass

    def setemergencyType(self, emergencyType):
        self.emergencyType = emergencyType

    def getemergencyType(self):
        return self.emergencyType

    def setname(self, name):
        self.name = name

    def getname(self):
        return self.name

    def setlatitude(self, latitude):
        self.latitude = str(latitude)

    def getlatitude(self):
        if self.latitude == '':
            return '0'
        else:

            return self.latitude

    def setlongitude(self, longitude):
        self.longitude = str(longitude)

    def getlongitude(self):
        if self.longitude == '':
            return '0'
        else:
            return self.longitude

if __name__ == "__main__":
    x = EmergencyPost()
    1 == 1


from FreeTAKServer.controllers.services.FederationServiceAbstract import FederationServiceAbstract
from FreeTAKServer.controllers.services.FederationServiceController import FederationServiceController
from FreeTAKServer.controllers.configuration.LoggingConstants import LoggingConstants
from FreeTAKServer.controllers.CreateLoggerController import CreateLoggerController
loggingConstants = LoggingConstants()
logger = CreateLoggerController("FederationClient").getLogger()
import ssl
import socket
from queue import Queue

class FederationClientServiceController(FederationServiceAbstract):
    """this controller is instantiated upon the startup of FTS and is used to
    establish and control federations. this is NOT always running and has no while loops
    """
    def __init__(self, pipe):
        super().__init__(pipe)
        self.dataQueue = Queue()

    def create_client_federation(self, ip, port):
        try:
            sock = self._establish_connection(ip, port)
            federate = FederationServiceController(ip, port, sock).start()
            self.federateClients.add_client(federate)
        except Exception as e:
            logger.error('exception has been thrown in the creation of client federation '
                         + str(e))

    def remove_client_federation(self, Federate):
        pass

    def _establish_connection(self, ip, port):
        try:
            """TODO: add certificate validation using supplied CA note: in future FTS to FTS
                federation should use more secure cyphers for communication
            """
            # this function loads the ssl certificates and waits for a connection over the specified port
            # set TLSv1.2 as the version for communication
            context = ssl.SSLContext(ssl.PROTOCOL_TLSv1_2)

            # load the required certificates for authentication
            context.load_cert_chain(
                '/home/natha/ProgrammerStuff/FTS/FTSDev/FreeTAKServer/Certs/ServerCerts/testalpha.pem',
                '/home/natha/ProgrammerStuff/FTS/FTSDev/FreeTAKServer/Certs/ServerCerts/testalpha.key',
                password='atakatak')

            # downgrade to cyphers supported on TAKServer
            context.set_ciphers('DEFAULT@SECLEVEL=1')
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock = sock.connect((ip, port))
            ssock = context.wrap_socket(sock, server_hostname=ip)
            return ssock

        except Exception as e:
            logger.error('there has been an exception thrown in '
                         'the establishment of a client federation ' + str(e))

    def send(self, data):
        try:
            """
            a wrapper so that pipe method names can be used to interact
            with the class
            :param data:
            :return:
            """
            self.send_data_to_clients(data, self.federateClients.get_all_clients())
        except Exception as e:
            logger.error('there has been an exception throw in the sending '
                         'of data to a server this instance is federated to ' + str(e))
            raise e
    def recv(self):
        try:
            """
            a wrapper so that pipe method names can be used to interact
            with the class
            :return:
            """
            self.recv_data_from_clients(self.federateClients.get_all_clients())
        except Exception as e:
            logger.error('there has been an exception thrown in the reception '
                         'of data to a server this instance is federated to ' + str(e))
            raise e
    def poll(self):
        try:
            """
            a wrapper so that pipe method names can be used to interact
            with the class it calls the check_if_data_is_available method
            within FederationServiceAbstract and sends the returned data in
            pipe in
            :return:
            """
            data = self.check_if_data_is_available(self.federateClients.get_all_clients())
            if len(data) > 0:
                for CoT in data:
                    self.dataQueue.put(CoT)
                return True

            else:
                raise multiprocessing.TimeoutError
        except Exception as e:
            logger.error('there has been an exception thrown in the polling '
                         'of data to a server this instance is federated to ' + str(e))
            raise e

    def recv_data_from_clients(self, clients):
        from FreeTAKServer.model.ServiceObjects import Federate
        try:
            if isinstance(clients, list):
                for client in clients:
                    if isinstance(client, Federate):
                        for CoT in client.federationController.receive_data_from_federates():
                            self.dataQueue.put(CoT)

                    else:
                        raise TypeError('the list should only contain instances of the Federate class')
                return 1
            else:
                raise TypeError('this function should only be supplied a list object')
        except Exception as e:
            return -1


if __name__ == "__main__":
    import multiprocessing
    from FreeTAKServer.controllers import SendOtherController
    from FreeTAKServer.model.RawCoT import RawCoT
    import time
    temp = RawCoT()
    temp.type = None
    temp.clientInformation = None
    temp.protocol = None
    temp.xmlString = b'<event version="2.0" uid="123e915c-7b9a-44ed-91e7-5165e384b213" type="a-h-G" time="2020-09-02T00:15:50.504Z" start="2020-09-02T00:15:50.504Z" stale="2020-09-02T00:20:50.504Z" how="h-g-i-g-o"><point lat="28.31507" lon="-81.34564" hae="-5.185734847652469" ce="6.0" le="9999999.0"/><detail><contact callsign="2340 Cordova Ct, Kissimmee, FL 34743, USA"/><status readiness="true"/><archive/><link uid="ANDROID-863134036519299" production_time="2020-07-19T18:32:31.535Z" type="a-f-G-E-V-C" parent_callsign="WP4JMV" relation="p-p"/><remarks/><archive/><ce_human_input>true</ce_human_input><color argb="-1"/><_flow-tags_ TAK-Server-c0a2d13e="2020-09-01T12:32:01Z"/></detail></event>'
    a, b = multiprocessing.Pipe()
    #multiprocessing.Process(target=).start()
    FtsObject = SendOtherController(temp).getObject()
    #b.send(FtsObject)
    FederationClientServiceController(a, '192.168.2.105', 9000).start()
    time.sleep(1000)
    x = 1
    '''
    event {
  sendTime: 1598963344000
  startTime: 1599005949000
  staleTime: 1599006324000
  lat: 43.855695
  lon: -66.108016
  hae: 13.058658054176142
  ce: 58.9
  le: 9999999.0
  uid: "ANDROID-359975090666199"
  type: "a-f-G-U-C"
  coordSource: "h-e"
  other: "<detail><__group name=\"Yellow\" role=\"Team Member\"/><takv device=\"SAMSUNG SM-G950W\" platform=\"ATAK-CIV\" os=\"28\" version=\"4.0.0.1 (3a23eb4e).1585690033-CIV\"/><contact endpoint=\"*:-1:stcp\" phone=\"19027743142\" callsign=\"NOVA\"/><uid Droid=\"NOVA\"/><_flow-tags_ TAK-Server-c0a2d13e=\"2020-09-01T12:29:04Z\"/></detail>"
  battery: 48
  ploc: "GPS"
  palt: "GPS"
  course: 176.05777346688942
    }   '''


class ChatVariables:
    def __init__(self):
        self.SENDERCALLSIGN = None
        self.ID = None
        self.PARENT = None
        self.CHATROOM = None
        self.GROUPOWNER = None

    @classmethod
    def geochat(cls):
        cls.SENDERCALLSIGN = 'SERVER-UID'
        cls.ID = "All Chat Rooms"
        cls.PARENT = None
        cls.CHATROOM = "All Chat Rooms"
        cls.GROUPOWNER = "false"
        return cls


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
#######################################################
# 
# IdentityTypes.py
# Python implementation of the Class IdentityTypes
# Generated by Enterprise Architect
# Created on(FTSProtocolObject):    11-Feb-2020 11(FTSProtocolObject):08(FTSProtocolObject):08 AM
# Original author: Corvo
# 
#######################################################


class IdentityTypes(FTSProtocolObject):
    # default constructor    def __init__(self,):  

    # pending
    pending = "p" 
    # pending getter 
    def getpending(self,): 
        return self.pending 
 
    # pending setter 
    def setpending(self,pending=0):  
        self.pending=pending 
 
    # unknown
    unknown = "u" 
    # unknown getter 
    def getunknown(self,): 
        return self.unknown 
 
    # unknown setter 
    def setunknown(self,unknown=0):  
        self.unknown=unknown 
 
    # friend
    friend = "f" 
    # friend getter 
    def getfriend(self,): 
        return self.friend 
 
    # friend setter 
    def setfriend(self,friend=0):  
        self.friend=friend 
 
    neutral = "n" 
    # neutral getter 
    def getneutral(self,): 
        return self.neutral 
 
    # neutral setter 
    def setneutral(self,neutral=0):  
        self.neutral=neutral 
 
    hostile = "h" 
    # hostile getter 
    def gethostile(self,): 
        return self.hostile 
 
    # hostile setter 
    def sethostile(self,hostile=0):  
        self.hostile=hostile 
 
    assumedfriend = "a" 
    # assumed-friend getter 
    def getassumedfriend(self,): 
        return self.assumedfriend 
 
    # assumed-friend setter 
    def setassumedfriend(self, assumedfriend=0):  
        self.assumedfriend=assumedfriend 
 
    suspect = "s" 
    # suspect getter 
    def getsuspect(self,): 
        return self.suspect 
 
    # suspect setter 
    def setsuspect(self,suspect=0):  
        self.suspect=suspect 
 
    joker = "j" 
    # joker getter 
    def getjoker(self,): 
        return self.joker 
 
    # joker setter 
    def setjoker(self,joker=0):  
        self.joker=joker 
 
    faker = "f" 
    # faker getter 
    def getfaker(self,): 
        return self.faker 
 
    # faker setter 
    def setfaker(self,faker=0):  
        self.faker=faker 
    


from .SendCoTAbstractController import SendCoTAbstractController
from FreeTAKServer.model.SpecificCoT.SendUserUpdate import SendUserUpdate

class SendUserUpdateController(SendCoTAbstractController):
    def __init__(self, RawCoT=None):
        try:
            tempObject = super().Event.UserUpdate()
            object = SendUserUpdate()
            if RawCoT is not None:
                self.fill_object(object, tempObject, RawCoT)
            else:
                pass
        except Exception as e:
            print('send user update exception ' + str(e))



from FreeTAKServer.model.SpecificCoT.SendEmergency import SendEmergency
from FreeTAKServer.controllers.configuration.LoggingConstants import LoggingConstants
from FreeTAKServer.controllers.CreateLoggerController import CreateLoggerController
from FreeTAKServer.model.RestMessages.RestEnumerations import RestEnumerations
from FreeTAKServer.model.FTSModel.Event import Event as event
import json as jsonmodule
from FreeTAKServer.controllers.XMLCoTController import XMLCoTController
from FreeTAKServer.controllers.DatabaseControllers.DatabaseController import DatabaseController
from FreeTAKServer.model.RestMessages.EmergencyPost import EmergencyPost, RestEnumerations
from FreeTAKServer.model.RestMessages.EmergencyDelete import EmergencyDelete
loggingConstants = LoggingConstants()
logger = CreateLoggerController("SendGeoChatController").getLogger()

class SendEmergencyController:
    def __init__(self, json):
        if isinstance(json, EmergencyPost):
            tempObject = event.emergecyOn()
        elif isinstance(json, EmergencyDelete):
            tempObject = event.emergecyOff()
        else:
            raise Exception('unsupported object type passed under json argument')
        object = SendEmergency()
        object.setModelObject(tempObject)
        object.modelObject = self._serializeJsonToModel(object.modelObject, json)
        object.setXmlString(XMLCoTController().serialize_model_to_CoT(object.modelObject))
        self.setCoTObject(object)

    def _serializeJsonToModel(self, object, json):
        # runs if emergency is on
        if isinstance(json, EmergencyPost):
            object.settype(RestEnumerations.emergencyTypes[json.getemergencyType()])
            object.detail.contact.setcallsign(json.getname())
            object.detail.emergency.settype(json.getemergencyType())
            object.point.setlat(json.getlatitude())
            object.point.setlon(json.getlongitude())
            DatabaseController().create_ActiveEmergency(object)
            return object

        # runs if emergency is off
        elif isinstance(json, EmergencyDelete):
            object.setuid(json.getuid())
            DatabaseController().remove_ActiveEmergency(query=f'uid == "{object.uid}"')
            object.settype('b-a-o-can')
            object.detail.emergency.setcancel('true')
            return object

    def setCoTObject(self, CoTObject):
        self.CoTObject = CoTObject

    def getCoTObject(self):
        return self.CoTObject

if __name__ == "__main__":
    SendEmergencyController('{"name": "test", "emergencyType": "911 Alert", "status": "on"}')
    SendEmergencyController('{"uid": "d7f30cf5-1d4d-11eb-b036-2cf05d092d98", "status": "off"}')


from sqlalchemy import Column, ForeignKey
from FreeTAKServer.model.SQLAlchemy.Root import Base, Root
from sqlalchemy import String
from sqlalchemy import Integer

class ExCheckKeywords(Base):
    __tablename__ = 'ExCheckKeywords'
    PrimaryKey = Column(ForeignKey("ExCheckData.PrimaryKey"), primary_key = True, autoincrement=True)
    name = Column(String)
    description = Column(String)
    callsign = Column(String)


from FreeTAKServer.model.ServiceObjects.SSLDataPackageVariables import SSLDataPackageVariables as vars
class SSLDataPackageService:
    def __init__(self, SSLDataPackageServiceIP = vars().SSLDataPackageIP,
                 SSLDataPackageServicePort = vars().SSLDataPackagePort,
                 SSLDataPackageServiceStatus = vars().SSLDataPackageStatus):
        self.SSLDataPackageServiceIP = SSLDataPackageServiceIP
        self.SSLDataPackageServicePort = SSLDataPackageServicePort
        self.SSLDataPackageServiceStatus = SSLDataPackageServiceStatus


import os
import pathlib
from FreeTAKServer.controllers.configuration.MainConfig import MainConfig

class DataPackageServerConstants:
    def __init__(self):
        # http server config
        self.DATABASE = 'FreeTAKServerDataPackageDataBase.db'
        self.APIPORT = '8080'
        self.DEFAULTRETURN = 'other'
        self.GET = 'GET'
        self.PUT = 'PUT'
        self.POST = 'POST'
        self.DATAPACKAGEFOLDER = 'FreeTAKServerDataPackageFolder'
        self.HTTPDEBUG = False
        self.HTTPMETHODS = ['POST', 'GET', 'PUT']
        self.IP = "0.0.0.0"
        self.versionInfo = MainConfig.version
        self.NodeID = 'FTS'
        self.VERSIONJSON = '{"version":"2","type":"ServerConfig", "data":{"version": "%s", "api": "2","hostname":"%s"}, "nodeId":"%s"}' % (
            self.versionInfo, "0.0.0.0", self.NodeID)



from FreeTAKServer.model.SQLAlchemy.Root import Base, Root
from sqlalchemy import Column, String, Integer, UniqueConstraint, Boolean

class Federations(Base):
    __tablename__ = "Federations"
    id = Column(String, primary_key=True)
    name = Column(String, unique=True)
    address = Column(String)
    port = Column(String)
    status = Column(String, default="Disabled")
    reconnectInterval = Column(Integer, default=30)
    maxRetries = Column(Integer, default=0)
    federate = Column(String, default="")
    protocolVersion = Column(Integer, default=1)
    connectionStatus = Column(String, default="DISABLED")
    lastError = Column(String, nullable=True)
    fallBack = Column(String, nullable=True)

class ActiveFederations(Base):
    __tablename__ = "ActiveFederations"
    id = Column(String, primary_key=True)
    federate = Column(String, default="")
    address = Column(String)
    port = Column(Integer)
    initiator = Column(String)
    readCount = Column(Integer, default=0)
    processedCount = Column(Integer, default=0)


#######################################################
# 
# ActiveThreadsController.py
# Python implementation of the Class ActiveThreadsController
# Generated by Enterprise Architect
# Created on:      21-May-2020 9:23:03 AM
# Original author: Natha Paquette
#
#######################################################
from FreeTAKServer.model.ActiveThreads import ActiveThreads

class ActiveThreadsController:
    def __init__(self):  
        self.ActiveThreads = ActiveThreads()

    def addClientThread(self, clientInformation, process):
        processObject = (clientInformation, process)
        self.ActiveThreads.ThreadArray.append(processObject)

    def addReceiveConnectionsThread(self, ReceiveConnectionsProcess, process):
        processObject = (ReceiveConnectionsProcess, process)
        self.ActiveThreads.ThreadArray.append(processObject)

    def removeClientThread(self, clientInformation):
        for x in self.ActiveThreads.ThreadArray:
            if x[0] == clientInformation:
                self.ActiveThreads.ThreadArray.remove(x)

    def removeReceiveConnectionProcess(self, ReceiveConnectionsProcess):
        for x in self.ActiveThreads.ThreadArray:
            if x[0] == ReceiveConnectionsProcess:
                self.ActiveThreads.ThreadArray.remove(x)



from FreeTAKServer.controllers.configuration.MainConfig import MainConfig

class SSLCoTServiceVariables:
    def __init__(self):
        self.SSLCoTServiceIP = "0.0.0.0"
        self.SSLCoTServicePort = MainConfig.SSLCoTServicePort
        self.SSLCoTServiceStatus = ""


#######################################################
# 
# Federate.py
# Python implementation of the Class Federate
# Generated by Enterprise Architect
# Created on:      29-Dec-2020 8:10:34 AM
# Original author: natha
# 
#######################################################
import socket

class Federate:
# default constructor  def __init__(self):  
    def __init__(self):
        self.name = str
        self.uid = str
        self.addr = tuple
        self.conn = socket.socket


class RemarksVariables:
    def __init__(self):
        self.TIME = None
        self.SOURCE = None
        self.SOURCEID = None
        self.INTAG = None

    @classmethod
    def drop_point(cls):
        cls.TIME = None
        cls.SOURCE = None
        cls.SOURCEID = None
        cls.INTAG = None
        return cls

    @classmethod
    def geochat(cls):
        cls.TIME = None
        cls.SOURCE = 'SERVER'
        cls.SOURCEID = None
        cls.INTAG = 'DEFAULT'
        cls.TO = 'All Chat Rooms'
        return cls


#######################################################
# 
# GeoObject.py
# Python implementation of the Class GeoObject
# Generated by Enterprise Architect
# Created on:      04-Nov-2020 4:27:04 PM
# Original author: natha
# 
#######################################################
from FreeTAKServer.model.RestMessages.RestEnumerations import RestEnumerations


class GeoObject:
    """a GeoObject is an element place on a map. It has a name, characteristics and an
    attitude.
    """
# default constructor  def __init__(self):  
    # the kind of expected behavior of the GeoObject (e.g friendly, hostile, unknown).
    # Please see API documentation for a list of valid entries.
    attitude = ""
    # a GeoObject is the information that will determine which type will be placed on
    # the tak maps including his icon. Please see API documentation for a list of
    # valid entries.
    geoObject = ""
    # the way in which this geo information has been acquired. Please see API
    # documentation for a list of valid entries.
    how = ""
    # the angular distance of the geoobject from the earths equator expressed in
    # positive or negative float. (e.g 43.855682)
    #
    #  remember to set the display of your TAK in decimal cohordinates, where *West
    # 77.08* is equal to '-77.08' in the API
    latitude = ""
    # the angular distance of the geoobject from the meridian of the greenwich, UK
    # expressed in positive or negative float. (e.g -76.107.7998)
    longitude = ""
    # A string to ID the GeoObject on a map.
    name = ""
    # the length, expressed in seconds  until the point will stale out. Default is
    # 300 seconds or 5 minutes.
    timeout = ""


from lxml import etree

class AbstractXMLSerializer:
    def __init__(self, xml):
        self.xml = xml

    def serializeFromXML(self):
        xml = etree.fromstring(self.xml)
        self.inTag(xml.text)
        for attribute in xml.attrib:
            self.attrib(attribute)
        for element in xml:
            self.tag(element)

    def tag(self, xml):
        pass

    def attrib(self, xml):
        pass

    def inTag(self, xml):
        pass


from FreeTAKServer.controllers.configuration.MainConfig import MainConfig

class template:
    def __init__(self):
        self.version = "2"
        self.type = "Mission"
        self.data = []
        self.nodeId = MainConfig.nodeID


from FreeTAKServer.model.RestMessages.EmergencyPost import EmergencyPost
from FreeTAKServer.model.RestMessages.EmergencyDelete import EmergencyDelete
from FreeTAKServer.model.RestMessages.PresencePost import PresencePost
from FreeTAKServer.model.RestMessages.ChatPost import ChatPost
from FreeTAKServer.model.RestMessages.GeoObjectPost import GeoObjectPost

class JsonController:
    def __init__(self):
        pass

    def serialize_emergency_post(self, json):
        """
        :arg json: the json to be serialized to an emergency
        """
        object = EmergencyPost()
        return self.serialize_json_to_object(object, json)

    def serialize_emergency_delete(self, json):
        object = EmergencyDelete()
        return self.serialize_json_to_object(object, json)

    def serialize_geoobject_post(self, json):
        object = GeoObjectPost()
        return self.serialize_json_to_object(object, json)

    def serialize_presence_post(self, json):
        object = PresencePost()
        return self.serialize_json_to_object(object, json)

    def serialize_chat_post(self, json):
        object = ChatPost()
        return self.serialize_json_to_object(object, json)

    def serialize_json_to_object(self, object, json):
        for key in json.keys():
            s = dir(object)
            if key in dir(object):

                setter = getattr(object, 'set'+str(key))
                setter(json[key])

            else:
                raise Exception(f'attribute {key} not supported by this endpoint')

        return object


from FreeTAKServer.controllers.serializers.api_adapters.api_adapters import AbstractApiAdapter, SpecificCoTAbstract
from typing import NewType
from FreeTAKServer.controllers.serializers.json_serializer import JsonSerializer
from FreeTAKServer.model.FTSModel.Event import Event
from FreeTAKServer.model.RestMessages.GeoObject import GeoObject, RestEnumerations
from FreeTAKServer.controllers.configuration.RestAPIVariables import RestAPIVariables
from FreeTAKServer.model.SpecificCoT.SendSimpleCoT import SendSimpleCoT
from FreeTAKServer.controllers.serializers.xml_serializer import XmlSerializer
from FreeTAKServer.controllers.configuration.types import Types
from lxml import etree

class GeoObjectAdapter(AbstractApiAdapter, JsonSerializer):

    def from_api_to_fts_object(self, api_json: dict) -> Types.specific_cot:

        serialized_json = self.serialize_initial_json(api_json)
        CoTObject = self.create_cot_object(serialized_json, Event.SimpleCoT(), SendSimpleCoT())
        return CoTObject

    def serialize_initial_json(self, api_json: dict) -> dict:
        api_json['how'] = RestEnumerations.how[api_json['how']]
        if "-.-" in RestEnumerations.geoObject[api_json["geoObject"]]:
            api_json['type'] = RestEnumerations.geoObject[api_json["geoObject"]].replace("a-.-", RestEnumerations.attitude[api_json['attitude']])
        else:
            raise Exception('geoObject not supported')
        del(api_json['attitude'])
        del(api_json['geoObject'])
        api_json['lon'] = api_json['longitude']
        del(api_json['longitude'])
        api_json['lat'] = api_json['latitude']
        del(api_json['latitude'])
        api_json['callsign'] = api_json['name']
        del(api_json['name'])
        import datetime as dt
        DATETIME_FMT = "%Y-%m-%dT%H:%M:%S.%fZ"
        timer = dt.datetime
        now = timer.utcnow()
        zulu = now.strftime(DATETIME_FMT)
        if 'timeout' in api_json:
            add = dt.timedelta(seconds=int(api_json['timeout']))
        else:
            import datetime as dt
            add = dt.timedelta(seconds=RestAPIVariables.defaultGeoObjectTimeout)
        stale_part = dt.datetime.strptime(zulu, DATETIME_FMT) + add
        api_json['stale'] = stale_part.strftime(DATETIME_FMT)
        del(api_json['timeout'])
        return api_json

    def create_cot_object(self, serialized_json: dict, expected_fts_obj: Event, final_cot_object: Types.specific_cot) -> Types.specific_cot:
        fts_object = super().from_format_to_fts_object(serialized_json, expected_fts_obj)
        CoTObject = final_cot_object
        CoTObject.setModelObject(fts_object)
        CoTObject.setXmlString(etree.tostring(XmlSerializer().from_fts_object_to_format(fts_object)))
        return CoTObject


#######################################################
# 
# DataQueueController.py
# Python implementation of the Class DataQueueController
# Generated by Enterprise Architect
# Created on:      20-May-2020 1:09:37 PM
# Original author: Natha Paquette
# 
#######################################################


class DataQueueController:
# default constructor  def __init__(self):  

    def addData(CoTObject):
        pass

    def addDataToAllQueues(CoTObject):
        pass

    def addDataToSpecificQueue(CoTObject):
        pass

    def checkPrivacy(CoTObject):
        pass


from FreeTAKServer.controllers.DatabaseControllers.table_controllers import TableController
from FreeTAKServer.controllers.DatabaseControllers.EventTableController import EventTableController
from FreeTAKServer.model.SQLAlchemy.ActiveEmergencys import ActiveEmergencys


class ActiveEmergencysController(TableController):
    def __init__(self):
        self.table = ActiveEmergencys
    
    def create(self, session, object):
        row = ActiveEmergencys()
        row.event = EventTableController().convert_model_to_row(object, row.event)
        session.add(row)
        session.commit()
        return row

if __name__ == "__main__":
    from FreeTAKServer.controllers.DatabaseControllers.DatabaseController import DatabaseController
    modelObject = SimulateSentCoT().createFullObj()
    #DatabaseController().create_engine()
    #session = DatabaseController().create_Session()
    #row = ActiveEmergencysController().create(session, modelObject)
    #ActiveEmergencysController().delete(session, row)


from FreeTAKServer.controllers.configuration.LoggingConstants import LoggingConstants
from logging.handlers import RotatingFileHandler
import logging
import os
loggingConstants = LoggingConstants()
class CreateLoggerController:
    def __init__(self, loggername):
        self.currentpath = str(os.path.dirname(os.path.realpath(__file__)))
        self.logger = logging.getLogger(loggername)
        self.logger.propagate = True
        log_format = logging.Formatter(loggingConstants.LOGFORMAT)
        self.logger.setLevel(logging.DEBUG)
        self.logger.addHandler(self.newHandler(loggingConstants.DEBUGLOG, logging.DEBUG, log_format))
        self.logger.addHandler(self.newHandler(loggingConstants.ERRORLOG, logging.ERROR, log_format))
        self.logger.addHandler(self.newHandler(loggingConstants.INFOLOG, logging.INFO, log_format))
        """console = logging.StreamHandler(sys.stdout)
        console.setFormatter(log_format)
        console.setLevel(logging.DEBUG)
        self.logger.info('test')
        self.logger.addHandler(console)"""

    def newHandler(self, filename, log_level, log_format):
        handler = RotatingFileHandler(
            filename,
            maxBytes=loggingConstants.MAXFILESIZE,
            backupCount=loggingConstants.BACKUPCOUNT
        )
        handler.setFormatter(log_format)
        handler.setLevel(log_level)
        return handler

    def getLogger(self):
        return self.logger


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
#######################################################
# 
# _medevac_ .py
# Python implementation of the Class _medevac_ 
# Generated by Enterprise Architect
# Created on(FTSProtocolObject):      13-Apr-2020 4(FTSProtocolObject):07(FTSProtocolObject):52 PM
# Original author: Corvo
# 
#######################################################

class _medevac_(FTSProtocolObject):
    """the medevac class is used to describe a case of someone in need to be evacuated
    """
# default constructor  def __init__(self):  

    __litter = "1" 
    __freq = "0.0" 
    __terrain_none = "true" 
    __zone_prot_selection = "0" 
    __Title = "" 
    __Priority = "1" 
    __medline_remarks = "" 
    __Security = "" 
    __routine = "" 
    __equipment_none = "" 
    __hlz_marking = "" 
    __casevac = "" 
    __urgent = ""

class _medevac_ :
    """the medevac class is used to describe a case of someone in need to be evacuated
    """
# default constructor  def __init__(self):  

    __litter = "1" 
    __freq = "0.0" 
    __terrain_none = "true" 
    __zone_prot_selection = "0" 
    __Title = "" 
    __Priority = "1" 
    __medline_remarks = "" 
    __Security = "" 
    __routine = "" 
    __equipment_none = "" 
    __hlz_marking = "" 
    __casevac = "" 
    __urgent = ""    


from FreeTAKServer.model.SpecificCoT.SendHealthCheck import SendHealthCheck

class SendHealthCheckController:
    # TODO: deprecate this function
    def __init__(self, RawCoT):
        self.RawCoT = RawCoT
        self.HealthCheck = SendHealthCheck()
        self.HealthCheck.xmlString = RawCoT.xmlString
        self.HealthCheck.clientInformation = RawCoT.clientInformation

    def getObject(self):
        return self.HealthCheck


from abc import ABC
from FreeTAKServer.controllers.services.service_abstracts import ServerServiceInterface, ServiceInterface
from FreeTAKServer.model.federate import Federate
from FreeTAKServer.model.clients import ClientAbstract
from FreeTAKServer.controllers.configuration.types import Types
from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
from FreeTAKServer.model.SpecificCoT.SpecificCoTAbstract import SpecificCoTAbstract
from FreeTAKServer.controllers.SpecificCoTControllers.SendCoTAbstractController import SendCoTAbstractController

class HandlerInterface(ABC):
    
    def Handle(self, obj, command):
        raise NotImplementedError

    def setNextHandler(self, handler):
        raise NotImplementedError

class HandlerBase(HandlerInterface):
    """ implements basic functionality required for all handlers

    this class implements the setNextHandler method from the HandlerInterface
    as well as creating a callNextHandler method which will either call the next
    handler in the chain of responsibility or raise an exception if there are no
    remaining handlers.
    """

    def __init__(self):
        self.nextHandler = None

    def Handle(self, obj: ServiceInterface, command):
        raise NotImplementedError
    
    def setNextHandler(self, handler: HandlerInterface):
        """ set next handler to be called in chain of responsibility

        :params handler: the handler which should be next in chain of responsibility
        """

        self.nextHandler = handler

    def callNextHandler(self, obj: ServiceInterface, command):
        if self.nextHandler is None:
            raise Exception("no further handlers, object not supported by responsibility chain")
        else:
            self.nextHandler.Handle(obj, command)

class ConnectHandler(HandlerBase):
    """ Handler for command to connect to new server

    """
    def Handle(self, obj: ServiceInterface, command):
        if isinstance(command, tuple) and command[1] == "CREATE":
            obj.connect_to_server(command)

        else:
            self.callNextHandler(obj, command)

class DisconnectHandler(HandlerBase):
    """ Handler for command to disconnect client

    """

    def Handle(self, obj: ServiceInterface, command):
        if isinstance(obj, ServerServiceInterface) and isinstance(command, tuple) and command[1] == "DELETE":
            obj.disconnect_client(command[0])

        else:
            self.callNextHandler(obj, command)

class SendDataHandler(HandlerBase):
    """ Handler for command send data to client

    """

    def Handle(self, obj: ServiceInterface, command: any):

        if isinstance(obj, ServerServiceInterface) and isinstance(command, SpecificCoTAbstract):
            obj.send_data_to_clients(command)

        else:
            self.callNextHandler(obj, command)

class SendConnectionDataHandler(HandlerBase):

    def Handle(self, obj: ServiceInterface, command: any):
        from FreeTAKServer.model.ClientInformation import ClientInformation

        if isinstance(obj, ServerServiceInterface) and isinstance(command, ClientInformation):
            obj.send_connection_data(command)
            return None
        else:
            self.callNextHandler(obj, command)

class SendDisconnectionDataHandler(HandlerBase):
    def Handle(self, obj: ServiceInterface, command):
        from FreeTAKServer.model.SpecificCoT.SendDisconnect import SendDisconnect

        if isinstance(obj, ServerServiceInterface) and isinstance(command, SendDisconnect):
            obj.send_disconnection_data(command)

        else:
            self.callNextHandler(obj, command)

class StopHandler(HandlerBase):
    """ Handler for command stop service

    """

    def Handle(self, obj: ServiceInterface, command):

        if isinstance(obj, ServiceInterface) and command == "STOP":
            obj.stop()

        else:
            self.callNextHandler(obj, command)


class EmergencyVariables:
    def __init__(self):
        self.TYPE = None
        self.ALERT = None
        # if true the Emergency beacon is canceled
        self.CANCEL = None

    @classmethod
    def emergency_on(cls):
        cls.CANCEL = None
        cls.ALERT = None
        cls.TYPE = None
        cls.INTAG = None
        return cls

    @classmethod
    def emergency_off(cls):
        cls.CANCEL = None
        cls.INTAG = None
        return cls


#######################################################
# 
# Precisionlocation.py
# Python implementation of the Class Precisionlocation
# Generated by Enterprise Architect
# Created on:      26-Sep-2020 9:41:37 PM
# Original author: natha
# 
#######################################################
from sqlalchemy import Column, ForeignKey
from FreeTAKServer.model.SQLAlchemy.Root import Base
from sqlalchemy import String
from sqlalchemy.orm import relationship


class Precisionlocation(Base):
# default constructor  def __init__(self):  

    __tablename__ = "Precisionlocation"
    PrimaryKey = Column(ForeignKey("Detail.PrimaryKey"), primary_key=True)
    Detail = relationship("Detail", back_populates="precisionlocation")
    # TDB can be DTED0 or ???
    altsrc = Column(String)
    geopointsrc = Column(String)


class RestAPIVariables:

    defaultGeoObjectTimeout = 300
    defaultPresenceTimeout = 500
    defaultPresenceType = "a-f-G-U-C-I"

    def __init__(self):
        pass

    def function_names(self):
        self.Status = 'start_all'
        self.Clients = 'show_users'
        self.checkStatus = 'check_server_status'

    def json_vars(self):
        self.COTSERVICE = 'TCP_CoT_service'
        self.DATAPACKAGESERVICE = 'TCP_DataPackage_service'
        self.SSLCOTSERVICE = 'SSL_CoT_service'
        self.SSLDATAPACKAGESERVICE = 'SSL_DataPackage_service'
        self.FEDERATIONCLIENTSERVICE = 'Federation_Client_service'
        self.FEDERATIONSERVERSERVICE = 'Federation_server_service'
        self.RESTAPISERVICE = 'Rest_API_service'
        self.IP = 'ip'
        self.PORT = 'port'
        self.STATUS = 'status'

    def rest_methods(self):
        self.POST = 'POST'
        self.GET = 'GET'
        self.DELETE = 'DELETE'

    def default_values(self):
        self.defaultIP = "127.0.0.1"
        self.defaultPort = 80
        self.defaultCoTIP = '0.0.0.0'
        self.defaultCoTPort = 15777
        self.defaultCoTStatus = 'start'
        self.defaultDataPackageIP = '0.0.0.0'
        self.defaultDataPackagePort = 8080
        self.defaultDataPackageStatus = 'start'
        self.defaultSSLCoTIP = '0.0.0.0'
        self.defaultSSLCoTPort = 15778
        self.defaultSSLCoTStatus = 'start'
        self.defaultFederationClientIP = '0.0.0.0'
        self.defaultFederationClientPort = 9000
        self.defaultFederationClientStatus = ''
        self.defaultFederationServerIP = '0.0.0.0'
        self.defaultFederationServerPort = 9000
        self.defaultFederationServerStatus = ''

    def json_content(self):
        self.JsonStatusStartAll = {"CoTService":
                                   {"IP": self.getdefaultCoTIP(), "PORT": self.getdefaultCoTPort(), "STATUS": self.getdefaultCoTStatus()},
                               "TCPDataPackageService":
                                   {"IP": self.getdefaultDataPackageIP(), "PORT": self.getdefaultDataPackagePort(), "STATUS": self.getdefaultDataPackageStatus()},
                                   "SSLCoTService":
                                       {"IP":self.getdefaultSSLCoTIP(), "PORT": self.getdefaultSSLCoTPort(), "STATUS": self.getdefaultSSLCoTStatus()},
                                   "FederationClientService":
                                       {"IP":self.getdefaultFederationClientIP(), "PORT": self.getdefaultFederationClientPort(), "STATUS": self.getdefaultFederationClientStatus()},
                                   "FederationServerService":
                                       {"IP":self.getdefaultFederationServerIP(), "PORT": self.getdefaultFederationServerPort(), "STATUS": self.getdefaultFederationServerStatus()}}
        self.JsonStatusStopAll = {"CoTService":
                                      {"STATUS": "stop"},
                                  "TCPDataPackageService":
                                      {"STATUS": "stop"},
                                  "SSLCoTService":
                                      {"STATUS": "stop"},
                                  "FederationClientService":
                                      {"STATUS": "stop"},
                                  "FederationServerService":
                                      {"STATUS": "stop"}}
        self.JsonStatusStartCoTService = {}

    def setJsonStatusStartAll(self, JsonStatusStartAll):
        self.JsonStatusStartAll = JsonStatusStartAll

    def getJsonStatusStartAll(self):
        self.json_content()
        return self.JsonStatusStartAll

    def setdefaultCoTIP(self, defaultCoTIP):
        self.defaultCoTIP = defaultCoTIP

    def getdefaultCoTIP(self):
        return self.defaultCoTIP

    def setdefaultCoTPort(self, defaultCoTPort):
        self.defaultCoTPort = defaultCoTPort

    def getdefaultCoTPort(self):
        return self.defaultCoTPort

    def setdefaultCoTStatus(self, defaultCoTStatus):
        self.defaultCoTStatus = defaultCoTStatus

    def getdefaultCoTStatus(self):
        return self.defaultCoTStatus

    def setdefaultSSLCoTPort(self, defaultSSLCoTPort):
        self.defaultSSLCoTPort = defaultSSLCoTPort

    def getdefaultSSLCoTPort(self):
        return self.defaultSSLCoTPort

    def setdefaultSSLCoTIP(self, defaultSSLCoTIP):
        self.defaultSSLCoTIP = defaultSSLCoTIP

    def getdefaultSSLCoTIP(self):
        return self.defaultSSLCoTIP

    def setdefaultSSLCoTStatus(self, defaultSSLCoTStatus):
        self.defaultSSLCoTStatus = defaultSSLCoTStatus

    def getdefaultSSLCoTStatus(self):
        return self.defaultSSLCoTStatus

    def setdefaultDataPackageIP(self, defaultDataPackageIP):
        self.defaultDataPackageIP = defaultDataPackageIP

    def getdefaultDataPackageIP(self):
        return self.defaultDataPackageIP

    def setdefaultDataPackagePort(self, defaultDataPackagePort):
        self.defaultDataPackagePort = defaultDataPackagePort

    def getdefaultDataPackagePort(self):
        return self.defaultDataPackagePort
    
    def setdefaultDataPackageStatus(self, defaultDataPackageStatus):
        self.defaultDataPackageStatus = defaultDataPackageStatus

    def getdefaultDataPackageStatus(self):
        return self.defaultDataPackageStatus
    
    def setdefaultFederationClientIP(self, defaultFederationClientIP):
        self.defaultFederationClientIP = defaultFederationClientIP

    def getdefaultFederationClientIP(self):
        return self.defaultFederationClientIP

    def setdefaultFederationClientPort(self, defaultFederationClientPort):
        self.defaultFederationClientPort = defaultFederationClientPort

    def getdefaultFederationClientPort(self):
        return self.defaultFederationClientPort
    
    def setdefaultFederationClientStatus(self, defaultFederationClientStatus):
        self.defaultFederationClientStatus = defaultFederationClientStatus

    def getdefaultFederationClientStatus(self):
        return self.defaultFederationClientStatus

    def setdefaultFederationServerIP(self, defaultFederationServerIP):
        self.defaultFederationServerIP = defaultFederationServerIP

    def getdefaultFederationServerIP(self):
        return self.defaultFederationServerIP

    def setdefaultFederationServerPort(self, defaultFederationServerPort):
        self.defaultFederationServerPort = defaultFederationServerPort

    def getdefaultFederationServerPort(self):
        return self.defaultFederationServerPort
    
    def setdefaultFederationServerStatus(self, defaultFederationServerStatus):
        self.defaultFederationServerStatus = defaultFederationServerStatus

    def getdefaultFederationServerStatus(self):
        return self.defaultFederationServerStatus



class TrackVariables:
    def __init__(self):
        self.COURSE = None
        self.SPEED = None

    @classmethod
    def connection(cls):
        cls.SPEED = None
        cls.COURSE = None
        return cls

    @classmethod
    def UserUpdate(cls):
        cls.speed = None
        cls.course = None
        return cls


from FreeTAKServer.controllers.configuration.MainConfig import MainConfig
from FreeTAKServer.controllers.configuration.DataPackageServerConstants import DataPackageServerConstants
import ssl
from FreeTAKServer.controllers.services.DataPackageServer import FlaskFunctions, Path, dp_directory, os, app, eventlet, const
from FreeTAKServer.controllers.configuration.LoggingConstants import LoggingConstants
from FreeTAKServer.controllers.CreateLoggerController import CreateLoggerController
from FreeTAKServer.controllers.SSLSocketController import SSLSocketController

loggingConstants = LoggingConstants()
logger = CreateLoggerController("SSLDataPackageServer").getLogger()

class SSLDataPackageService(FlaskFunctions):
    def startup(self, ip, port, pipe):
        try:
            from eventlet import wsgi, wrap_ssl, listen
            import socket
            import ssl
            from FreeTAKServer.controllers.MainSocketController import MainSocketController
            from FreeTAKServer.model.sockets.SSLServerSocket import SSLServerSocket
            global IP, HTTPPORT
            self.MainSocket = SSLServerSocket()
            IP = ip
            HTTPPORT = port
            PIPE = pipe
            # Make sure the data package directory exists
            if not Path(dp_directory).exists():
                app.logger.info(f"Creating directory at {str(dp_directory)}")
                os.makedirs(str(dp_directory))
            # Create the relevant database tables
            print(const.IP)
            print(HTTPPORT)
            super().setIP(IP)
            super().setHTTPPORT(HTTPPORT)
            super().setPIPE(PIPE)
            #wsgi.server(eventlet.listen(('', 14533)), app)  keyfile=MainConfig.keyDir,
            self.SSLSocketController = SSLSocketController()
            self.SSLSocketController.changeIP(IP)
            self.SSLSocketController.changePort(HTTPPORT)

            wsgi.server(sock=wrap_ssl(listen((DataPackageServerConstants().IP, int(HTTPPORT))), keyfile=MainConfig.unencryptedKey,
                                      certfile=MainConfig.pemDir,
                                      server_side=True,ca_certs=MainConfig.CA), site=app)
        except Exception as e:
            logger.error('there has been an exception in Data Package service startup ' + str(e))
            return -1
        
if __name__ == "__main__":
    SSLDataPackageService().startup('0.0.0.0', 8443)


from multiprocessing import TimeoutError
from FreeTAKServer.controllers.FederatedCoTController import FederatedCoTController
from FreeTAKServer.model.FTSModel.Event import Event
from FreeTAKServer.model.protobufModel.fig_pb2 import FederatedEvent
import codecs
import socket
from FreeTAKServer.model.ServiceObjects.Federate import Federate

class FederationServiceController:
    def __init__(self, ip, port, connection):
        self.ip = ip
        self.port = port
        self.pool = None
        self.buffer = 4
        self.excessData = None
        self.killSwitch = False
        self.connection = connection


    def start(self):

        federateObject = Federate()
        federateObject.federationController = self
        federateObject.IP = self.connection.getpeername()[0]
        federateObject.Port = self.connection.getpeername()[1]
        federateObject.Socket = self.connection
        return federateObject

    def establish_connection(self, ip, port):
        # returns a socket object
        pass

    def disconnect(self):
        try:
            self.connection.shutdown(socket.SHUT_RDWR)
            self.connection.close()
            return 1
        except:
            self.connection.close()
            return 1

    def generate_header(self, contentlength):
        tempHex = format(contentlength, 'x')
        if (len(tempHex) % 2) == 0:
            filteredhex = [(tempHex[i:i + 2]) for i in range(0, len(tempHex), 2)]
            while len(filteredhex) < 4:
                filteredhex.insert(0, '00')
            filteredhex = r'\x'.join(filteredhex)
            filteredhex = r'\x' + filteredhex
            filteredhex = codecs.escape_decode(filteredhex)[0]
            return filteredhex
        else:
            tempHex = '0'+tempHex
            filteredhex = [(tempHex[i:i + 2]) for i in range(0, len(tempHex), 2)]
            while len(filteredhex) < 4:
                filteredhex.insert(0, '00')
            filteredhex = r'\x'.join(filteredhex)
            filteredhex = r'\x' + filteredhex
            filteredhex = codecs.escape_decode(filteredhex)[0]
            return filteredhex

    def get_header_length(self, header):
        try:
            from binascii import hexlify
            headerInHex = header.split(b'\\x')
            if len(headerInHex[-1]) == 3:
                headerInHex[-1][2] = hexlify(headerInHex[-1][2])
            headerInHex = b''.join(headerInHex)
            return int(headerInHex, 16)
        except:
            return -1

    def receive_data_from_federates(self):
        # returns data received from federate
        # the following logic receives data from the federate and processes the protobuf
        # up to 100 CoT's
        dataCount = 0
        dataArray = []
        # 100 is the limit of data which can be received from a federate in one iteration
        while dataCount < 100:
            dataCount += 1
            try:
                try:
                    self.connection.settimeout(0.01)
                    data = self.connection.recv(self.buffer)
                    self.connection.settimeout(0)
                except TimeoutError:
                    break

                except Exception as e:
                    self.disconnect()
                    self.killSwitch = True
                    return 0
                if data != [b'']:
                    header = data[0]
                    content = self.connection.recv(self.get_header_length(header))
                    EmptyFTSObject = Event.FederatedCoT()
                    protoObject = FederatedEvent().FromString(content)
                    print(protoObject)
                    FTSObject = FederatedCoTController().serialize_main_contentv1(protoObject, EmptyFTSObject)
                    print('received data from Federate')
                    print(content)
                else:
                    self.killSwitch = True

                dataArray.append(FTSObject)

            except Exception as e:
                pass

    def send_data_to_federates(self, data):
        try:

            # sends supplied data to supplied socket upon being called
            federatedEvent = FederatedEvent()
            ProtoObj = FederatedCoTController().serialize_from_FTS_modelv1(federatedevent=federatedEvent, ftsobject=data)
            protostring = ProtoObj.SerializeToString()
            header = self.generate_header(len(protostring))
            protostring = header + protostring
            print(b'sent '+protostring+b' to federate')
            self.connection.send(protostring)
            return 1

        except Exception as e:
            pass

    def recv_in_data_pipe(self, pipe):
        pass

    def send_in_data_pipe(self, pipe, data):
        pass
if __name__ == '__main__':
    FederationServiceController
    content = b'\n\xb1\x06\x08\xd8\x81\xa6\xa1\xec.\x10\x84\xf0\xa5\xa1\xec.\x18\x84\xa8\xbf\xca\xec.1\x00\x00\x00\xe0\xcf\x12cA9\x00\x00\x00\xe0\xcf\x12cAA\x00\x00\x00\xe0\xcf\x12cAJjGeoChat.S-1-5-21-2720623347-3037847324-4167270909-1002.All Chat Rooms.673a0aa4-c8eb-4bb5-aa7e-abab7d8d89a0R\x05b-t-fZ\th-g-i-g-ob\x80\x05<detail><__chat id="All Chat Rooms" chatroom="All Chat Rooms" senderCallsign="FEATHER" groupOwner="false"><chatgrp id="All Chat Rooms" uid0="S-1-5-21-2720623347-3037847324-4167270909-1002" uid1="All Chat Rooms"/></__chat><link uid="S-1-5-21-2720623347-3037847324-4167270909-1002" type="a-f-G-U-C-I" relation="p-p"/><remarks source="BAO.F.WinTAK.S-1-5-21-2720623347-3037847324-4167270909-1002" sourceID="S-1-5-21-2720623347-3037847324-4167270909-1002" to="All Chat Rooms" time="2021-01-02T17:33:40.74Z">aa</remarks><_flow-tags_ TAK-Server-c0581fed97ff4cb89eb8666a8794670cc9f77ddb-badf-48da-abe7-84545ecda69d="2021-01-02T17:33:43Z"/></detail>'
    y = FederatedEvent().FromString(content)
    1==1


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
from FreeTAKServer.model.FTSModelVariables.DestVariables import DestVariables as vars

class Dest(FTSProtocolObject):
    def __init__(self, CALLSIGN = vars.other().CALLSIGN):
        self.callsign = None
        self.__settercalled = False
        self.__gettercalled = False
        # self.setcallsign(CALLSIGN)

    @staticmethod
    def other(CALLSIGN = vars.geochat().CALLSIGN):
        dest = Dest()

        return dest

    @staticmethod
    def geochat(CALLSIGN = vars.geochat().CALLSIGN):
        dest = Dest()
        # dest.setcallsign(CALLSIGN)
        return dest

    def getcallsign(self):
        self.__gettercalled = True
        return self.callsign

    def setcallsign(self, callsign=0):
        self.callsign = callsign
        self.__settercalled = True

    def _settercalled(self):
        return self.__settercalled

    def _gettercalled(self):
        return self.__gettercalled


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
from FreeTAKServer.model.FTSModel.Contact import Contact

class Group(FTSProtocolObject):
    def __init__(self, xml):
        self.uid = None
        self.name = None
        self.setuid(xml.get('uid'))
        self.setname(xml.get('name'))
        try:
            self.contact = Contact()
        except:
            pass
        try:
            self.Group = Group(xml.find('group'))
        except:
            pass

    def setuid(self, uid=None):
        self.uid = uid
        
    def getuid(self):
        return self.uid

    def setname(self, name=None):
        self.name = name

    def getname(self):
        return self.name


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: detail.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import FreeTAKServer.model.protobufModel.contact_pb2 as contact__pb2
import FreeTAKServer.model.protobufModel.group_pb2 as group__pb2
import FreeTAKServer.model.protobufModel.precisionlocation_pb2 as precisionlocation__pb2
import FreeTAKServer.model.protobufModel.status_pb2 as status__pb2
import FreeTAKServer.model.protobufModel.takv_pb2 as takv__pb2
import FreeTAKServer.model.protobufModel.track_pb2 as track__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='detail.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0c\x64\x65tail.proto\x1a\rcontact.proto\x1a\x0bgroup.proto\x1a\x17precisionlocation.proto\x1a\x0cstatus.proto\x1a\ntakv.proto\x1a\x0btrack.proto\"\xc1\x01\n\x06\x44\x65tail\x12\x11\n\txmlDetail\x18\x01 \x01(\t\x12\x19\n\x07\x63ontact\x18\x02 \x01(\x0b\x32\x08.Contact\x12\x15\n\x05group\x18\x03 \x01(\x0b\x32\x06.Group\x12-\n\x11precisionLocation\x18\x04 \x01(\x0b\x32\x12.PrecisionLocation\x12\x17\n\x06status\x18\x05 \x01(\x0b\x32\x07.Status\x12\x13\n\x04takv\x18\x06 \x01(\x0b\x32\x05.Takv\x12\x15\n\x05track\x18\x07 \x01(\x0b\x32\x06.Trackb\x06proto3'
  ,
  dependencies=[contact__pb2.DESCRIPTOR,group__pb2.DESCRIPTOR,precisionlocation__pb2.DESCRIPTOR,status__pb2.DESCRIPTOR,takv__pb2.DESCRIPTOR,track__pb2.DESCRIPTOR,])




_DETAIL = _descriptor.Descriptor(
  name='Detail',
  full_name='Detail',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='xmlDetail', full_name='Detail.xmlDetail', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='contact', full_name='Detail.contact', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='group', full_name='Detail.group', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='precisionLocation', full_name='Detail.precisionLocation', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='status', full_name='Detail.status', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='takv', full_name='Detail.takv', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='track', full_name='Detail.track', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=109,
  serialized_end=302,
)

_DETAIL.fields_by_name['contact'].message_type = contact__pb2._CONTACT
_DETAIL.fields_by_name['group'].message_type = group__pb2._GROUP
_DETAIL.fields_by_name['precisionLocation'].message_type = precisionlocation__pb2._PRECISIONLOCATION
_DETAIL.fields_by_name['status'].message_type = status__pb2._STATUS
_DETAIL.fields_by_name['takv'].message_type = takv__pb2._TAKV
_DETAIL.fields_by_name['track'].message_type = track__pb2._TRACK
DESCRIPTOR.message_types_by_name['Detail'] = _DETAIL
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Detail = _reflection.GeneratedProtocolMessageType('Detail', (_message.Message,), {
  'DESCRIPTOR' : _DETAIL,
  '__module__' : 'detail_pb2'
  # @@protoc_insertion_point(class_scope:Detail)
  })
_sym_db.RegisterMessage(Detail)


# @@protoc_insertion_point(module_scope)



from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
#######################################################
# 
# color.py
# Python implementation of the Class color
# Generated by Enterprise Architect
# Created on(FTSProtocolObject):      11-Feb-2020 11(FTSProtocolObject):08(FTSProtocolObject):07 AM
# Original author: Corvo
# 
#######################################################
from FreeTAKServer.model.FTSModelVariables.ColorVariables import ColorVariables as vars

class Color(FTSProtocolObject):
    def __init__(self):
        self.argb = None

    @staticmethod
    def drop_point(ARGB = vars.drop_point().ARGB):
        color = Color()
        color.setargb(ARGB)
        return color

    # argb getter 
    def getargb(self): 
        return self.argb 

    # argb setter 
    def setargb(self, argb=0):  
        self.argb=argb 



from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
from FreeTAKServer.model.FTSModelVariables.UidVariables import UidVariables as vars

class Uid(FTSProtocolObject):
    def __init__(self):
        self.Droid = None

    @staticmethod
    def connection(DROID = vars.connection().DROID):
        uid = Uid()
        uid.setDroid(DROID)
        return uid

    @staticmethod
    def UserUpdate(DROID=vars.UserUpdate().Droid):
        uid = Uid()
        uid.setDroid(DROID)
        return uid

    @staticmethod
    def ExcheckUpdate(INTAG = vars.ExcheckUpdate().INTAG):
        uid = Uid()
        uid.setINTAG(INTAG)
        return uid

    @staticmethod
    def Checklist(INTAG = vars.Checklist().INTAG):
        uid = Uid()
        uid.setINTAG(INTAG)
        return uid

    def getDroid(self):
        return self.Droid

    def setDroid(self, Droid = None):
        self.Droid = Droid

    def getINTAG(self):
        return self.INTAG

    def setINTAG(self, INTAG = None):
        self.INTAG = INTAG






from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
from FreeTAKServer.model.FTSModel.MissionChange import MissionChange

class MissionChanges(FTSProtocolObject):
    def __init__(self):
        pass

    @staticmethod
    def ExcheckUpdate():
        missionchanges = MissionChanges()
        missionchanges.MissionChange = MissionChange.ExcheckUpdate()
        return missionchanges




import shutil
from sqlalchemy.sql import text

class TableController:
    # default constructor  def __init__(self):
    def __init__(self):
        self.table = None


    def delete(self, session, query):
        # this function removes a row from the datapackages table based on the uid
        session.query(self.table).filter(text(query)).delete(synchronize_session='fetch')
        session.commit()
        return 1

    def create(self, session, **args):
        # this function adds a new row to the datapackages table
        session.add(self.table(**args))
        session.commit()


    def query(self, session, query, columns):
        # query needs to be applicable to datapackage object tuple(['DataPackage.'+x for x in columns])
        output = session.query(*tuple([getattr(self.table, x) if x != '*' else self.table for x in columns])).filter(
            text(query)).all()
        return output


    def update(self, session, query, column_value):
        DataPackages = session.query(self.table).filter(
            text(query)).all()  # self.query(session, query, [column for column, value in column_value.items()])
        for dp in DataPackages:
            for column, value in column_value.items():
                setattr(dp, column, value)
        session.commit()


#######################################################
# 
# FederateClients.py
# Python implementation of the Class FederateClients
# Generated by Enterprise Architect
# Created on:      12-Sep-2020 10:32:03 PM
# Original author: natha
# 
#######################################################

class FederateClients:
    def __init__(self):
        # list of instances of the Federate Object.
        self.__clientArray = []

    def add_client(self, Federate):
        """add client to federate clients list
        """
        self.__clientArray.append(Federate)

    def remove_client(self, Federate):
        self.__clientArray.remove(Federate)

    def get_all_clients(self):
        return self.__clientArray


import socket
from time import sleep, gmtime, strftime
import uuid
import smtplib, ssl
import time

class statusCheck:
    def __init__(self):
        # !/usr/bin/env python3
        # needs python 3.x due to some different functions

        # Healthchecker for FTS TAK self.servers. May work with others, but not tested
        # Alan Barrow July 16, 2020 alan@pinztrek.com
        self.version = "1.0"

        # tune your params below, then execute. It will run until interupted.
        # prints "+" each time it checks a server.
        # Will only work with self.servers not using secure connections




        # --------------------------------------------------------------------------------------------------
        # Setup our test params

        # How long to sleep between checks in seconds
        self.sleeptime = 60 * 2

        # Setup Email params
        # multiple recips separated by comma and a space, format is pretty picky

        self.recip = []

        self.sender = ''

        # Your smtp server
        self.smtp_server = ''

        # Your smtp port. Should be 465 or similar SSL, the sendmail is setup to use SSL
        self.smtp_port = ''

        # The account email to authorize with. Sometimes need a + instead of @
        self.smtp_acct = ""

        # Password to send email
        self.smtp_password = ""

        self.smtp_trace_level = 0  # 1 is useful trace

        # List of self.servers to monitor
        self.servers = [
            ["server", "ip", "port"]
        ]

        # -----------------------------------------------------------------------------------------
        # Should not have to modify below here

        # Setup a UID
        self.my_uid = str(socket.getfqdn())
        self.my_uid = self.my_uid + "-" + str(uuid.uuid1())
        self.my_uid = bytes("healthchecker-" + self.my_uid, "UTF-8")

        # get a time, does not really matter
        self.xml_time = bytes(strftime("%Y-%m-%dT%H:%M:%SZ", gmtime()), "UTF-8")

        # XML strings for the healthcheck
        self.dataString = b'<?xml self.version="1.0" encoding="UTF-8" standalone="yes"?>\n<event self.version="2.0" uid="' + self.my_uid + b'" type="t-x-c-t" time="' + self.xml_time + b'" start="' + self.xml_time + b'" stale="' + self.xml_time + b'" how="h-g-i-g-o"><point lat="0.0" lon="0.0" hae="9999999.0" ce="9999999.0" le="9999999.0"/><detail><takv platform="healthcheck" os="24" self.version="1.0.0.0"/><contact/></detail></event>'

        self.connectionString = self.dataString

        # Uncomment to see the xml sent
        # print(self.dataString)

        # -------------------------------------------------------------------------
        # Make these global
        self.status = ""
        self.sock = ''

        # Get rid of any old messages, really should not be needed with unique UID's
    def flushit(self):
        self.sock.settimeout(1)

        # response = 'start some reading'
        # Flush any pending server responses
        while True:
            # print("Read attempt: " + str(i))
            try:
                response = self.sock.recv(2048)
                # print("flushit response is:")
                # print(response)
                pass

            except:
                # print("flushit read empty")
                # Flushed, now return
                break

        # return response

    def readit(self):
        self.sock.settimeout(1)

        response = ''
        # Flush any pending server responses
        # while True:
        for i in range(5):
            # print("Read attempt: " + str(i))
            try:
                response = self.sock.recv(2048)
                # print("readit response is:")
                # print(response)
                return response

            except Exception as e:
                # print("readit read empty")
                pass

        return response

    def closeit(self):
        # print("shutting down")
        self.sock.shutdown(1)
        time.sleep(0.1)
        self.sock.close()

    def notify(self, status):
        # Log event to the screen
        print("")  # since the progress dots do not have new line

        # uncomment to just print the event, not the email. (and comment out the email print)
        # print(strftime("%d/%m/%y %H:%M ", gmtime()) + self.status)
        # Add timestamp
        status = strftime("%d/%m/%y %H:%M ", gmtime()) + status

        smtp_subject = status
        smtp_body = status
        my_recips = ""
        for cnt in range(len(self.recip) - 1):
            my_recips = my_recips + self.recip[cnt] + ", "
        my_recips = my_recips + self.recip[-1]

        # Useful when debugging multiple debugging, ok to comment out
        print("Recip String: " + my_recips)

        # Prepare the message package
        message = "From: " + self.sender + "\r\n" \
                  + "To: " + my_recips + "\r\n" \
                  + "Subject: " + smtp_subject \
                  + "\r\n\r\n" + smtp_body \
 \
            # useful when debugging email
        print(message)

        # Blast the email
        context = ssl.create_default_context()
        with smtplib.SMTP_SSL(self.smtp_server, self.smtp_port, context=context) as server:
            try:
                server.set_debuglevel(self.smtp_trace_level)
                server.login(self.smtp_acct, self.smtp_password)
                server.sendmail(self.sender, self.recip, message)
                server.quit()
                print("Mail sent")
            except Exception as e:
                print(e)
                print("Mail send failed")

    def start(self):
        print("------------------------------------------------------------------------------")
        print("FTS Healthchecker Version " + self.version)
        while True:
            # print("starting loop again")
            for server in range(0, len(self.servers)):

                # print("server # is " + str(server))

                # Setup our locals
                testserver = self.servers[server][0]
                testip = self.servers[server][1]
                testport = self.servers[server][2]
                # print(str(server+1) + " " + testserver + " IP: " + testip + ":" + str(testport))

                # Open the socket
                self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

                # Longer timeout for busy self.servers
                self.sock.settimeout(10)

                # Now try to open
                try:
                    self.sock.connect((testip, testport))
                    # print("Connected: " + testip + ":" + str(testport))

                except socket.error as e:
                    # print(testserver + 'connection failed')
                    self.closeit()
                    self.notify(testserver + ": Connect failed socket error-" + str(e))
                    continue

                except:
                    # return 'connection failed'
                    # print(testserver + 'connection failed')
                    # No need to close, not open
                    # closeit()
                    self.notify(testserver + ": Connect failed")
                    continue

                # self.sock.send(b'')

                # Send Connect String to server
                # print("Now Connect string")
                try:
                    sent = self.sock.send(self.connectionString)
                except:
                    print(testserver + ": Connection send failed")
                    self.closeit()
                    self.notify(testserver + ": Connection send Failed")
                    continue

                # print("Connect response:")
                # Server sends active CoT's upon connect
                junk = self.flushit()  # Don't care about recent points
                # print(junk)

                # time.sleep(0.1)
                # flush the reads
                # junk=flushit()

                # print("Now send data string:")
                # print(self.dataString)
                sent = self.sock.send(self.dataString)

                # print("read the heartbeat response:")
                # Set Default response if it does not match
                # response = testserver + ": Healthcheck compare failed"
                response = ''

                # Do multiple checks in case server sends a valid CoT while testing
                for count in range(1, 20):
                    junk = self.readit()
                    # print("Heartbeat response " + str(count) + ":")
                    # print(junk)
                    if junk == self.dataString:
                        # print(testserver + ": OK")
                        # notify(testserver + ": OK")
                        response = testserver + ": OK"
                        # Exit the response check for loop
                        break
                    else:
                        response = testserver + ": Healthcheck compare failed"
                        # print(testserver + ": Healthcheck compare failed")
                        # pass
                # print("Cleanup and try next server")
                # Cleanup and notify failed check
                try:
                    self.closeit()
                except:
                    pass
                # Now send the response
                if response:
                    self.notify(response)

                # print("test")
                print("+", end="", flush=True)
                # print("Hello there!", end = '')
                # print(".")

            # print("+")
            # print("sleeping " + str(self.sleeptime))
            sleep(self.sleeptime)
if __name__ == "__main__":
    statusCheck().start()


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
from FreeTAKServer.model.FTSModel.ContentResource import ContentResource
from FreeTAKServer.model.FTSModel.CreatorUid import CreatorUid
from FreeTAKServer.model.FTSModel.MissionName import MissionName
from FreeTAKServer.model.FTSModel.Timestamp import Timestamp
from FreeTAKServer.model.FTSModel.Type import Type

class MissionChange(FTSProtocolObject):
    def __init__(self):
        pass
    @staticmethod
    def ExcheckUpdate():
        missionchange = MissionChange()
        missionchange.contentResource = ContentResource.ExcheckUpdate()
        missionchange.creatorUid = CreatorUid.ExcheckUpdate()
        missionchange.missionName = MissionName.ExcheckUpdate()
        missionchange.timestamp = Timestamp.ExcheckUpdate()
        missionchange.type = Type.ExcheckUpdate()
        return missionchange




import unittest

from FreeTAKServer.controllers.serializers.json_serializer import JsonSerializer
from FreeTAKServer.controllers.serializers.xml_serializer import XmlSerializer
from FreeTAKServer.controllers.serializers.protobuf_serializer import ProtobufSerializer
from FreeTAKServer.controllers.XMLCoTController import XMLCoTController
from FreeTAKServer.controllers.JsonController import JsonController
from FreeTAKServer.controllers.RestMessageControllers.SendSimpleCoTController import SendSimpleCoTController
from FreeTAKServer.controllers.RestMessageControllers.SendChatController import SendChatController
#from FreeTAKServer.controllers.RestMessageControllers.SendEmergencyController import SendEmergencyController
from lxml import etree

class TestSerializers(unittest.TestCase):

    def test_json_serializer_with_geoobject_adapter(self):
        """
        test new serialization in contrast with legacy serializer to ensure compatibility
        """
        from FreeTAKServer.model.FTSModel.Event import Event
        from FreeTAKServer.controllers.serializers.api_adapters.api_adapters import GeoObjectAdapter
        basejson = {
                        "attitude": "friendly",
                        "how": "nonCoT",
                        "name": "testing",
                        "latitude": "37.5677889",
                        "longitude": "12.345678",
                        "geoObject": "Ground",
                        "timeout": "120"
                    }
        jsonobj = JsonController().serialize_geoobject_post(basejson)
        simpleCoTObject = SendSimpleCoTController(jsonobj).getCoTObject()
        xml_legacy = etree.fromstring(simpleCoTObject.xmlString)
        xml_legacy.attrib['uid'] = '123'
        xml_legacy.attrib['time'] = '24'
        xml_legacy.attrib['start'] = '849'
        xml_legacy.attrib['stale'] = '3333'
        newObject = GeoObjectAdapter().from_api_to_fts_object(basejson)
        xml_updated = etree.fromstring(newObject.xmlString)
        xml_updated.attrib['uid'] = '123'
        xml_updated.attrib['time'] = '24'
        xml_updated.attrib['start'] = '849'
        xml_updated.attrib['stale'] = '3333'
        xml_legacy = etree.tostring(xml_legacy).decode()
        xml_updated = etree.tostring(xml_updated).decode()
        self.assertEqual(xml_legacy, xml_updated)

    def test_json_serializer_with_chat_adapter(self):
        from FreeTAKServer.model.FTSModel.Event import Event
        from FreeTAKServer.controllers.serializers.api_adapters.api_adapters import ChatAdapter
        basejson = {
                        "message": "testing",
                        "sender": "apiuser"
                    }
        jsonobj = JsonController().serialize_chat_post(basejson)
        simpleCoTObject = SendChatController(jsonobj).getCoTObject()
        xml_legacy = etree.fromstring(simpleCoTObject.xmlString)
        xml_legacy.attrib['uid'] = '123'
        xml_legacy.attrib['time'] = '24'
        xml_legacy.attrib['start'] = '849'
        xml_legacy.attrib['stale'] = '3333'
        newObject = ChatAdapter().from_api_to_fts_object(basejson)
        xml_updated = etree.fromstring(newObject.xmlString)
        xml_updated.attrib['uid'] = '123'
        xml_updated.attrib['time'] = '24'
        xml_updated.attrib['start'] = '849'
        xml_updated.attrib['stale'] = '3333'
        xml_legacy = etree.tostring(xml_legacy).decode()
        xml_updated = etree.tostring(xml_updated).decode()
        self.assertEqual(xml_legacy, xml_updated)

    def test_json_serializer_with_presence_adapter(self):
        from FreeTAKServer.model.FTSModel.Event import Event
        from FreeTAKServer.controllers.RestMessageControllers.SendPresenceController import SendPresenceController
        from FreeTAKServer.controllers.serializers.api_adapters.api_adapters import PresenceAdapter
        basejson = {
                        "how": "nonCoT",
                        "name": "testing",
                        "latitude": "34.5677889",
                        "longitude": "12.345678",
                        "role": "Team Member",
                        "team": "Cyan"
                    }
        jsonobj = JsonController().serialize_presence_post(basejson)
        simpleCoTObject = SendPresenceController(jsonobj).getCoTObject()
        xml_legacy = etree.fromstring(simpleCoTObject.xmlString)
        xml_legacy.attrib['uid'] = '123'
        xml_legacy.attrib['time'] = '24'
        xml_legacy.attrib['start'] = '849'
        xml_legacy.attrib['stale'] = '3333'
        newObject = PresenceAdapter().from_api_to_fts_object(basejson)
        xml_updated = etree.fromstring(newObject.xmlString)
        xml_updated.attrib['uid'] = '123'
        xml_updated.attrib['time'] = '24'
        xml_updated.attrib['start'] = '849'
        xml_updated.attrib['stale'] = '3333'
        xml_legacy = etree.tostring(xml_legacy).decode()
        xml_updated = etree.tostring(xml_updated).decode()
        self.assertEqual(xml_legacy, xml_updated)

    def test_json_serializer_with_emergency_adapter(self):
        from FreeTAKServer.model.FTSModel.Event import Event
        from FreeTAKServer.controllers.RestMessageControllers.SendEmergencyController import SendEmergencyController
        from FreeTAKServer.controllers.serializers.api_adapters.api_adapters import EmergencyOnAdapter
        basejson = {
                        "emergencyType": "911 Alert",
                        "name": "test"
                    }
        jsonobj = JsonController().serialize_emergency_post(basejson)
        EmergencyObject = SendEmergencyController(jsonobj).getCoTObject()
        xml_legacy = etree.fromstring(EmergencyObject.xmlString)
        xml_legacy.attrib['uid'] = '123'
        xml_legacy.attrib['time'] = '24'
        xml_legacy.attrib['start'] = '849'
        xml_legacy.attrib['stale'] = '3333'
        newObject = EmergencyOnAdapter().from_api_to_fts_object(basejson)
        xml_updated = etree.fromstring(newObject.xmlString)
        xml_updated.attrib['uid'] = '123'
        xml_updated.attrib['time'] = '24'
        xml_updated.attrib['start'] = '849'
        xml_updated.attrib['stale'] = '3333'
        xml_legacy = etree.tostring(xml_legacy).decode()
        xml_updated = etree.tostring(xml_updated).decode()
        self.assertEqual(xml_legacy, xml_updated)

    def test_xml_serialization(self):
        """
        test new serialization in contrast with legacy serializer to ensure compatibility
        """
        from FreeTAKServer.model.FTSModel.Event import Event
        xmlstring = '<event version="2.0" uid="GeoChat.ANDROID-359975090666199.FEATHER.27d8ef23-8578-4cb4-8f53-02f5dc150cd2" type="b-t-f" how="h-g-i-g-o" start="2021-01-03T19:01:35.472Z" time="2021-01-03T19:01:35.472Z" stale="2021-01-04T19:01:35.472Z"><detail><__chat id="S-1-5-21-2720623347-3037847324-4167270909-1002" parent="RootContactGroup" chatroom="FEATHER" groupOwner="false"><chatgrp uid0="ANDROID-359975090666199" uid1="S-1-5-21-2720623347-3037847324-4167270909-1002" id="S-1-5-21-2720623347-3037847324-4167270909-1002"/></__chat><link uid="ANDROID-359975090666199" relation="p-p" type="a-f-G-E-V-A"/><remarks time="2021-01-03T19:01:35.472Z" source="BAO.F.ATAK.ANDROID-359975090666199" to="S-1-5-21-2720623347-3037847324-4167270909-1002">at VDO</remarks><__serverdestination destinations="192.168.2.66:4242:tcp:ANDROID-359975090666199"/><marti><dest callsign = "WOLF"/><dest callsign="GALLOP"/><dest callsign="FEATHER"/></marti></detail><point le="9999999.0" ce="3.2" hae="22.958679722315807" lon="-66.10803" lat="43.855711"/></event>'
        fts_obj = XmlSerializer().from_format_to_fts_object(xmlstring, Event.GeoChat())
        obj = XmlSerializer().from_fts_object_to_format(fts_obj)
        print(etree.tostring(obj).decode())
        legacyftsobj = XMLCoTController().serialize_CoT_to_model(Event.GeoChat(), etree.fromstring(xmlstring))
        legacy_string = XMLCoTController().serialize_model_to_CoT(legacyftsobj)
        self.assertEqual(legacy_string.decode(), etree.tostring(obj).decode())

    def test_from_protobuf_serialization(self):
        from FreeTAKServer.model.FTSModel.Event import Event
        from FreeTAKServer.model.protobufModel.fig_pb2 import FederatedEvent
        import lxml
        protobufString = b'\n\x83\x04\x08\xbe\xb9\xfb\x85\xeb.\x10\xbe\xb9\xfb\x85\xeb.\x18\x96\xab\x92\x86\xeb.!\xaf\xf6b\x88\\d<@) {Mcr\x1aG\xc01\x00\x00\x00\xe0\xcf\x12cA9\x00\x00\x00\xe0\xcf\x12cAA\x00\x00\x00\xe0\xcf\x12cAJ.S-1-5-21-2720623347-3037847324-4167270909-1002R\x0ba-f-G-U-C-IZ\x03h-eb\xfa\x02<detail><takv version="4.0.0.117" platform="WinTAK-CIV" os="Microsoft Windows 10 Home" device="Micro-Star International Co., Ltd MS-7C02"/><contact callsign="FEATHER" endpoint="*:-1:stcp"/><uid Droid="FEATHER"/><__group name="Cyan" role="Team Member"/><_flow-tags_ TAK-Server-c0581fed97ff4cb89eb8666a8794670cc9f77ddb-badf-48da-abe7-84545ecda69d="2020-12-29T23:04:27Z"/></detail>hd'
        # protobufString2 = b'"\x00\x00\x00="\x12;\x08\x01\x12.S-1-5-21-2720623347-3037847324-4167270909-1002\x1a\x07FEATHER'
        protobufObj = FederatedEvent()
        protobufObj.ParseFromString(protobufString)
        # protobufObj.ParseFromString(protobufString2)
        #y = [name.name for name in protobufObj.event.DESCRIPTOR.fields]
        fts_obj = ProtobufSerializer().from_format_to_fts_object(protobufObj, Event.Connection())
        z = lxml.etree.tostring(XmlSerializer().from_fts_object_to_format(fts_obj))
        1 == 1

    def test_to_protobuf_serialization(self):
        from FreeTAKServer.model.FTSModel.Event import Event
        xmlstring = '<event version="2.0" uid="ANDROID-R5CN70EYKQH" type="a-f-G-U-C" how="h-e" start="2020-12-24T18:16:22.325Z" time="2020-12-24T18:16:22.325Z" stale="2020-12-24T18:22:37.325Z"><detail><__group name="Teal" role="Team Member"/><status battery="76"/><takv version="4.2.0.4 (47e136dd).1607456856-CIV" platform="ATAK-CIV" device="SAMSUNG SM-N986U" os="29"/><track course="159.1462509079387" speed="0.0"/><contact callsign="SPAC3SLOTH" endpoint="*:-1:stcp" /><uid Droid="SPAC3SLOTH"/><precisionlocation altsrc="GPS" geopointsrc="GPS"/></detail><point le="9999999.0" ce="11.0" hae="178.84407323983876" lon="-76.675505" lat="39.664392"/></event>'
        fts_obj = XmlSerializer().from_format_to_fts_object(xmlstring, Event.Connection())
        y = ProtobufSerializer().from_fts_object_to_format(fts_obj)
        1 == 1


class userInterfaceController:
    def __init__(self):
        self.commandDict = {
            "1" : "checkAliveProcesses",
            "2" : "getSocketInformation"
        }

    def userInput(self):
        command = str(input('FTS $ '))
        commandFunction = getattr(self, self.commandDict[command])
        commandFunctionOutput = commandFunction()

    def checkAliveProcesses(self):
        pass





class MissionVariables:

    @classmethod
    def ExcheckUpdate(cls):
        cls.TYPE = 'CHANGE'
        cls.TOOL = "ExCheck"
        cls.NAME = None
        cls.AUTHORUID = None
        return cls




from sqlalchemy import Column
from FreeTAKServer.model.SQLAlchemy.Root import Base
from sqlalchemy import String
from sqlalchemy import Boolean
from flask_login import UserMixin

class SystemUser(Base, UserMixin):

    __tablename__ = 'SystemUser'
    uid = Column(String(25), primary_key=True)
    name = Column(String(15), nullable=False)
    token = Column(String(30), nullable=True)
    password = Column(String(30), nullable=True)
    group = Column(String(15), default=True, nullable=True)
    certificate_package_name = Column(String(30), nullable=True, default=None)

    def __init__(self, **kwargs):
        for property, value in kwargs.items():
            # depending on whether value is an iterable or not, we must
            # unpack it's value (when **kwargs is request.form, some values
            # will be a 1-element list)
            if hasattr(value, '__iter__') and not isinstance(value, str):
                # the ,= unpack of a singleton fails PEP8 (travis flake8 test)
                value = value[0]

            setattr(self, property, value)

    def get_id(self):
        return self.uid

    def __repr__(self):
        return str(self.name)


from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
from FreeTAKServer.model.FTSModelVariables.MimeTypeVariables import MimeTypeVariables as vars
class MimeType(FTSProtocolObject):
    def __init__(self):
        self.INTAG = None

    @staticmethod
    def ExcheckUpdate(INTAG = vars.ExcheckUpdate().INTAG):
        mimetype = MimeType()
        mimetype.setINTAG(INTAG)
        return mimetype

    def setINTAG(self, INTAG):
        self.INTAG = INTAG

    def getINTAG(self):
        return self.INTAG


from .RestAPIServiceVariables import RestAPIServiceVariables as vars

class RestAPIService:
    def __init__(self, IP = vars().RestAPIServiceIP, Port = vars().RestAPIServicePort, Status = vars().RestAPIServiceStatus):
        self.RestAPIServiceIP = IP
        self.RestAPIServicePort = Port
        self.RestAPIServiceStatus = Status
        